@@ -48,7 +48,7 @@ DO_COMMAND(do_alias)
 	}
 	else if (*arg2 == 0)
 	{
-		if (show_node_with_wild(ses, arg1, LIST_ALIAS) == FALSE)
+		if (show_node_with_wild(ses, arg1, ses->list[LIST_ALIAS]) == FALSE)
 		{
 			show_message(ses, LIST_ALIAS, "#ALIAS: NO MATCH(ES) FOUND FOR {%s}.", arg1);
 		}
@@ -74,7 +74,7 @@ int check_all_aliases(struct session *ses, char *input)
 {
 	struct listnode *node;
 	struct listroot *root;
-	char line[BUFFER_SIZE], tmp[BUFFER_SIZE], *arg;
+	char tmp[BUFFER_SIZE], *arg;
 	int i;
 
 	root = ses->list[LIST_ALIAS];
@@ -84,46 +84,51 @@ int check_all_aliases(struct session *ses, char *input)
 		return FALSE;
 	}
 
-	for (i = 1 ; i < 100 ; i++)
-	{
-		if (*gtd->vars[i])
-		{
-			RESTRING(gtd->vars[i], "");
-		}
-	}
-
-	substitute(ses, input, line, SUB_VAR|SUB_FUN);
+//	substitute(ses, input, line, SUB_VAR|SUB_FUN);
 
 	for (root->update = 0 ; root->update < root->used ; root->update++)
 	{
-		if (check_one_regexp(ses, root->list[root->update], line, line, PCRE_ANCHORED))
+		if (check_one_regexp(ses, root->list[root->update], input, input, PCRE_ANCHORED))
 		{
 			node = root->list[root->update];
 
 			i = strlen(node->left);
 
-			if (!strncmp(node->left, line, i))
+			if (!strncmp(node->left, input, i))
 			{
-				if (line[i] && line[i] != ' ')
+				if (input[i] && input[i] != ' ')
 				{
 					continue;
 				}
 				
-				arg = get_arg_in_braces(ses, line, tmp, FALSE);
+				arg = get_arg_in_braces(ses, input, tmp, FALSE);
 
 				RESTRING(gtd->vars[0], arg)
 
-				for (i = 1 ; i < 100 && *arg ; i++)
+				for (i = 1 ; i < 100 ; i++)
 				{
 					arg = get_arg_in_braces(ses, arg, tmp, FALSE);
 
 					RESTRING(gtd->vars[i], tmp);
+
+					if (*arg == 0)
+					{
+						while (++i < 100)
+						{
+							if (*gtd->vars[i])
+							{
+								RESTRING(gtd->vars[i], "");
+							}
+						}
+						break;
+					}
+
 				}
 			}
 
 			substitute(ses, node->right, tmp, SUB_ARG);
 
-			if (!strncmp(node->left, line, strlen(node->left)) && !strcmp(node->right, tmp) && *gtd->vars[0])
+			if (!strncmp(node->left, input, strlen(node->left)) && !strcmp(node->right, tmp) && *gtd->vars[0])
 			{
 				sprintf(input, "%s %s", tmp, gtd->vars[0]);
 			}
