@@ -37,13 +37,13 @@ struct tintin_data *gtd;
 
 void pipe_handler(int signal)
 {
-	restore_terminal();
+//	restore_terminal();
 
-	clean_screen(gtd->ses);
+//	clean_screen(gtd->ses);
 
-	tintin_printf(NULL, "broken_pipe: dumping stack");
+	tintin_printf(NULL, "#SOCKET ERROR: RECEIVED SIGPIPE.");
 
-	dump_stack();
+//	dump_stack();
 }
 
 /*
@@ -116,18 +116,18 @@ void interrupt_handler(int signal)
 
 void suspend_handler(int signal)
 {
-	printf("\033[r\033[%d;%dH", gtd->ses->rows, 1);
+	printf("\e[r\e[%d;%dH", gtd->ses->rows, 1);
 
-	fflush(stdout);
+	fflush(NULL);
 
 	restore_terminal();
 
 	kill(0, SIGSTOP);
 
-	dirty_screen(gtd->ses);
-
 	init_terminal();
 
+	dirty_screen(gtd->ses);
+
 	tintin_puts(NULL, "#RETURNING BACK TO TINTIN++.");
 }
 
@@ -162,6 +162,7 @@ int main(int argc, char **argv)
 {
 	int greeting = TRUE;
 	char filename[256];
+	char arg[BUFFER_SIZE];
 
 	#ifdef SOCKS
 		SOCKSinit(argv[0]);
@@ -186,7 +187,7 @@ int main(int argc, char **argv)
 	{
 		syserr("signal SIGTERM");
 	}
-
+/*
 	if (signal(SIGINT, interrupt_handler) == BADSIG)
 	{
 		syserr("signal SIGINT");
@@ -196,7 +197,7 @@ int main(int argc, char **argv)
 	{
 		syserr("signal SIGSTOP");
 	}
-
+*/
 	if (signal(SIGPIPE, pipe_handler) == BADSIG)
 	{
 		syserr("signal SIGPIPE");
@@ -207,22 +208,19 @@ int main(int argc, char **argv)
 		syserr("signal SIGWINCH");
 	}
 
-
-	srand(time(NULL));
-
 	if (argc > 1)
 	{
 		int c;
 
-		while ((c = getopt(argc, argv, "e: G h r: s: t: v")) != EOF)
+		while ((c = getopt(argc, argv, "a: e: G h r: s: t: v")) != EOF)
 		{
-			if (c == 'G')
+			switch (c)
 			{
-				greeting = FALSE;
+				case 'G':
+					greeting = FALSE;
+					break;
 			}
 		}
-
-		optind = 1;
 	}
 
 	init_tintin(greeting);
@@ -245,10 +243,14 @@ int main(int argc, char **argv)
 
 		optind = 1;
 
-		while ((c = getopt(argc, argv, "e: G h r: s: t: v")) != EOF)
+		while ((c = getopt(argc, argv, "a: e: G h r: s: t: v")) != EOF)
 		{
 			switch (c)
 			{
+				case 'a':
+					strcpy(arg, optarg);
+					break;
+
 				case 'e':
 					gtd->quiet++;
 					gtd->ses = script_driver(gtd->ses, LIST_COMMAND, optarg);
@@ -261,6 +263,7 @@ int main(int argc, char **argv)
 				case 'h':
 					tintin_printf(NULL, "Usage: %s [OPTION]... [FILE]...", argv[0]);
 					tintin_printf(NULL, "");
+					tintin_printf(NULL, "  -a  Set argument for PROGRAM START event.");
 					tintin_printf(NULL, "  -e  Execute given command.");
 					tintin_printf(NULL, "  -G  Don't show the greeting screen.");
 					tintin_printf(NULL, "  -h  This help section.");
@@ -277,11 +280,7 @@ int main(int argc, char **argv)
 					break;
 
 				case 't':
-					printf("\033]0;%s\007", optarg);
-					break;
-
-				case 's':
-					srand((unsigned int) atoll(optarg));
+					printf("\e]0;%s\007", optarg);
 					break;
 
 				case 'v':
@@ -300,9 +299,8 @@ int main(int argc, char **argv)
 		}
 	}
 
-
-	check_all_events(gts, SUB_ARG|SUB_SEC, 0, 2, "PROGRAM START", CLIENT_NAME, CLIENT_VERSION);
-	check_all_events(gts, SUB_ARG|SUB_SEC, 0, 2, "SCREEN RESIZE", ntos(gts->cols), ntos(gts->rows));
+	check_all_events(gts, SUB_ARG, 0, 3, "PROGRAM START", CLIENT_NAME, CLIENT_VERSION, arg);
+	check_all_events(gts, SUB_ARG, 0, 2, "SCREEN RESIZE", ntos(gts->cols), ntos(gts->rows));
 
 	mainloop();
 
@@ -322,18 +320,22 @@ void init_tintin(int greeting)
 
 	gts->name           = strdup("gts");
 	gts->group          = strdup("");
-	gts->host           = strdup("");
-	gts->port           = strdup("");
+	gts->session_host   = strdup("");
+	gts->session_port   = strdup("");
 	gts->cmd_color      = strdup("");
 	gts->telopts        = TELOPT_FLAG_ECHO;
 	gts->flags          = SES_FLAG_MCCP;
 	gts->socket         = 1;
 	gts->read_max       = 16384;
+	gts->lognext_name   = strdup("");
+	gts->logline_name   = strdup("");
 
 	gtd                 = (struct tintin_data *) calloc(1, sizeof(struct tintin_data));
 
 	gtd->ses            = gts;
 
+	gtd->flags          = TINTIN_FLAG_INHERITANCE;
+
 	gtd->str_size       = sizeof(struct str_data);
 	gtd->str_hash_size  = sizeof(struct str_hash_data);
 
@@ -346,6 +348,7 @@ void init_tintin(int greeting)
 	gtd->input_off      = 1;
 
 	gtd->home           = strdup(getenv("HOME") ? getenv("HOME") : ".");
+	gtd->lang           = strdup(getenv("LANG") ? getenv("LANG") : "UNKNOWN");
 	gtd->term           = strdup(getenv("TERM") ? getenv("TERM") : "UNKNOWN");
 
 	for (index = 0 ; index < 100 ; index++)
@@ -368,22 +371,27 @@ void init_tintin(int greeting)
 
 	init_screen_size(gts);
 
-	printf("\033="); // set application keypad mode
+	init_local(gts);
+
+	printf("\e="); // set application keypad mode
 
 	gtd->input_level++;
 
 	do_configure(gts, "{AUTO TAB}         {5000}");
 	do_configure(gts, "{BUFFER SIZE}     {20000}");
+	do_configure(gts, "{COLOR MODE}       {AUTO}");
 	do_configure(gts, "{COLOR PATCH}       {OFF}");
 	do_configure(gts, "{COMMAND COLOR}   {<078>}");
 	do_configure(gts, "{COMMAND ECHO}       {ON}");
-	do_configure(gts, "{CONNECT RETRY}      {15}");
-	do_configure(gts, "{CHARSET}         {ASCII}");
+	do_configure(gts, "{CONNECT RETRY}       {0}");
+	do_configure(gts, "{CHARSET}          {AUTO}");
 	do_configure(gts, "{HISTORY SIZE}     {1000}");
 	do_configure(gts, "{LOG}               {RAW}");
 	do_configure(gts, "{PACKET PATCH}     {0.00}");
+	do_configure(gts, "{RANDOM SEED}      {AUTO}");
 	do_configure(gts, "{REPEAT CHAR}         {!}");
 	do_configure(gts, "{REPEAT ENTER}      {OFF}");
+	do_configure(gts, "{SCREEN READER}     {OFF}");
 	do_configure(gts, "{SCROLL LOCK}        {ON}");
 	do_configure(gts, "{SPEEDWALK}         {OFF}");
 	do_configure(gts, "{TINTIN CHAR}         {#}");
@@ -391,7 +399,6 @@ void init_tintin(int greeting)
 	do_configure(gts, "{VERBATIM CHAR}      {\\}");
 	do_configure(gts, "{VERBOSE}           {OFF}");
 	do_configure(gts, "{WORDWRAP}           {ON}");
-	do_configure(gts, "{256 COLORS}       {AUTO}");
 
 	gtd->input_level--;
 
@@ -413,7 +420,18 @@ void init_tintin(int greeting)
 	{
 		do_advertise(gts, "");
 
-		do_help(gts, "GREETING");
+		if (gts->cols >= 80)
+		{
+			do_help(gts, "GREETING");
+		}
+		else
+		{
+			tintin_printf2(gts,
+				"\e[0;37mT I N T I N + +   %s"
+				"\n\n\e[0;36mT\e[0;37mhe K\e[0;36mi\e[0;37mcki\e[0;36mn\e[0;37m \e[0;36mT\e[0;37mickin D\e[0;36mi\e[0;37mkuMUD Clie\e[0;36mn\e[0;37mt\n\n"
+				"Code by Peter Unold, Bill Reis, and Igor van den Hoven\n",
+				CLIENT_VERSION);
+		}
 	}
 }
 
@@ -434,7 +452,7 @@ void quitmsg(char *message)
 		close(gtd->chat->fd);
 	}
 
-	check_all_events(gts, SUB_ARG|SUB_SEC, 0, 0, "PROGRAM TERMINATION");
+	check_all_events(gts, SUB_ARG, 0, 0, "PROGRAM TERMINATION");
 
 	if (gtd->history_size)
 	{
@@ -449,13 +467,14 @@ void quitmsg(char *message)
 
 	clean_screen(gts);
 
-	if (message)
+	if (message == NULL || *message)
 	{
-		printf("\n%s\n", message);
+		if (message)
+		{
+			printf("\n%s", message);
+		}
+		printf("\nGoodbye from TinTin++\n\n");
 	}
-
-	printf("\nGoodbye from TinTin++\n\n");
-
 	fflush(NULL);
 
 	exit(0);
