@@ -535,7 +535,7 @@ DO_COMMAND(do_pathdir)
 	}
 	else if (*arg2 == 0)
 	{
-		if (show_node_with_wild(ses, arg1, LIST_PATHDIR) == FALSE)
+		if (show_node_with_wild(ses, arg1, ses->list[LIST_PATHDIR]) == FALSE)
 		{
 			show_message(ses, LIST_PATHDIR, "#NO MATCH(ES) FOUND FOR {%s}.", arg1);
 		}
@@ -555,11 +555,12 @@ DO_COMMAND(do_pathdir)
 		}
 		else
 		{
-			if (atoi(arg3) < 0 || atoi(arg3) >= 64)
+			if (!is_math(ses, arg3) || get_number(ses, arg3) < 0 || get_number(ses, arg3) >= 64)
 			{
 				show_message(ses, LIST_PATHDIR, "#PATHDIR: THE THIRD ARGUMENT MUST BE A NUMBER BETWEEN 0 and 63.");
 				return ses;
 			}
+			get_number_string(ses, arg3, arg3);
 		}
 		update_node_list(ses->list[LIST_PATHDIR], arg1, arg2, arg3);
 
@@ -568,10 +569,25 @@ DO_COMMAND(do_pathdir)
 	return ses;
 }
 
-
 DO_COMMAND(do_unpathdir)
 {
-	delete_node_with_wild(ses, LIST_PATHDIR, arg);
+	char arg1[BUFFER_SIZE];
+
+	arg = sub_arg_in_braces(ses, arg, arg1, GET_ONE, SUB_VAR|SUB_FUN);
+
+	do
+	{
+		if (delete_nest_node(ses->list[LIST_PATHDIR], arg1))
+		{
+			show_message(ses, LIST_VARIABLE, "#OK. {%s} IS NO LONGER A PATHDIR.", arg1);
+		}
+		else
+		{
+			delete_node_with_wild(ses, LIST_VARIABLE, arg1);
+		}
+		arg = sub_arg_in_braces(ses, arg, arg1, GET_ONE, SUB_VAR|SUB_FUN);
+	}
+	while (*arg1);
 
 	return ses;
 }
