@@ -47,7 +47,7 @@ DO_COMMAND(do_prompt)
 	}
 	else if (*arg1 && *arg2 == 0)
 	{
-		if (show_node_with_wild(ses, arg1, LIST_PROMPT) == FALSE)
+		if (show_node_with_wild(ses, arg1, ses->list[LIST_PROMPT]) == FALSE)
 		{
 			show_message(ses, LIST_PROMPT, "#PROMPT: NO MATCH(ES) FOUND FOR {%s}.", arg1);
 		}
@@ -140,7 +140,7 @@ void do_one_prompt(struct session *ses, char *prompt, int row)
 
 	if (len == 0)
 	{
-		sprintf(temp, "%.*s", ses->cols + 4, "\033[0m--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------");
+		sprintf(temp, "%.*s", ses->cols + 4, "\e[0m--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------");
 	}
 	else if (len <= ses->cols)
 	{
@@ -162,13 +162,13 @@ void do_one_prompt(struct session *ses, char *prompt, int row)
 	{
 		gtd->input_off = len + 1;
 
-		printf("\033[%d;1H\033[%d;1H\033[K%s%s\033[%d;%dH\0337\033[%d;1H", row, row, temp, gtd->input_buf, row, gtd->input_off + gtd->input_cur, ses->bot_row);
+		printf("\e[%d;1H\e[%d;1H\e[K%s%s\e[%d;%dH\e7\e[%d;1H", row, row, temp, gtd->input_buf, row, gtd->input_off + gtd->input_cur, ses->bot_row);
 	}
 	else
 	{
 		// goto row, erase to eol, print prompt, goto bot_row
 
-		printf("\033[%d;1H\033[%d;1H\033[K%s\033[%d;1H", row, row, temp, ses->bot_row);
+		printf("\e[%d;1H\e[%d;1H\e[K%s\e[%d;1H", row, row, temp, ses->bot_row);
 	}
 
 	if (!HAS_BIT(ses->flags, SES_FLAG_READMUD) && IS_SPLIT(ses))
