@@ -69,6 +69,7 @@ struct command_type command_table[] =
 	{    "killall",           do_kill,              TOKEN_TYPE_COMMAND },
 	{    "line",              do_line,              TOKEN_TYPE_COMMAND },
 	{    "list",              do_list,              TOKEN_TYPE_COMMAND },
+	{    "local",             do_local,             TOKEN_TYPE_COMMAND },
 	{    "log",               do_log,               TOKEN_TYPE_COMMAND },
 	{    "loop",              do_nop,               TOKEN_TYPE_LOOP    },
 	{    "macro",             do_macro,             TOKEN_TYPE_COMMAND },
@@ -79,6 +80,7 @@ struct command_type command_table[] =
 	{    "parse",             do_nop,               TOKEN_TYPE_PARSE   },
 	{    "path",              do_path,              TOKEN_TYPE_COMMAND },
 	{    "pathdir",           do_pathdir,           TOKEN_TYPE_COMMAND },
+	{    "port",              do_port,              TOKEN_TYPE_COMMAND },
 	{    "prompt",            do_prompt,            TOKEN_TYPE_COMMAND },
 	{    "read",              do_read,              TOKEN_TYPE_COMMAND },
 	{    "regexp",            do_regexp,            TOKEN_TYPE_REGEX   },
@@ -140,9 +142,8 @@ struct list_type list_table[LIST_MAX] =
 	{    "PATH",              "PATHS",              APPEND,      2,  LIST_FLAG_MESSAGE                                                                  },
 	{    "PATHDIR",           "PATHDIRS",           ALPHA,       3,  LIST_FLAG_MESSAGE|LIST_FLAG_READ|LIST_FLAG_WRITE|LIST_FLAG_CLASS|LIST_FLAG_INHERIT },
 	{    "PROMPT",            "PROMPTS",            PRIORITY,    3,  LIST_FLAG_MESSAGE|LIST_FLAG_READ|LIST_FLAG_WRITE|LIST_FLAG_CLASS|LIST_FLAG_INHERIT },
-	{    "SUBSTITUTE",        "SUBSTITUTIONS",      PRIORITY,    3,  LIST_FLAG_MESSAGE|LIST_FLAG_READ|LIST_FLAG_WRITE|LIST_FLAG_CLASS|LIST_FLAG_INHERIT },
+	{    "SUBSTITUTE",        "SUBSTITUTES",        PRIORITY,    3,  LIST_FLAG_MESSAGE|LIST_FLAG_READ|LIST_FLAG_WRITE|LIST_FLAG_CLASS|LIST_FLAG_INHERIT },
 	{    "TAB",               "TABS",               ALPHA,       1,  LIST_FLAG_MESSAGE|LIST_FLAG_READ|LIST_FLAG_WRITE|LIST_FLAG_CLASS|LIST_FLAG_INHERIT },
-//	{    "TABCYCLE",          "TABCYCLE",           APPEND,      1,  LIST_FLAG_MESSAGE|LIST_FLAG_HIDE                                                   },
 	{    "TICKER",            "TICKERS",            ALPHA,       3,  LIST_FLAG_MESSAGE|LIST_FLAG_READ|LIST_FLAG_WRITE|LIST_FLAG_CLASS|LIST_FLAG_INHERIT },
 	{    "VARIABLE",          "VARIABLES",          ALPHA,       2,  LIST_FLAG_MESSAGE|LIST_FLAG_READ|LIST_FLAG_WRITE|LIST_FLAG_CLASS|LIST_FLAG_INHERIT|LIST_FLAG_NEST }
 };
@@ -158,7 +159,8 @@ struct substitution_type substitution_table[] =
 	{    "SECURE",              64  },
 	{    "EOL",                128  },
 	{    "LNF",                256  },
-//	{    "FIX",               1024  },
+//	{    "FIX",                512  },
+        {    "COMPRESS",          1024  },
 	{    "",                  0     }
 };
 
@@ -185,6 +187,13 @@ struct config_type config_table[] =
 		config_charset
 	},
 
+	{
+		"COLOR MODE",
+		"",
+		"The color code encoding used by TinTin++",
+		config_colormode
+	},
+
 	{
 		"COLOR PATCH",
 		"TinTin++ will properly color the start of each line",
@@ -195,7 +204,7 @@ struct config_type config_table[] =
 	{
 		"CONNECT RETRY",
 		"",
-		"Seconds TinTin++ will try to connect before giving up",
+		"Seconds TinTin++ sessions try to connect on failure.",
 		config_connectretry
 	},
 
@@ -236,6 +245,13 @@ struct config_type config_table[] =
 		config_historysize
 	},
 
+	{
+		"INHERITANCE",
+		"The startup session is inherited",
+		"The startup session is not inherited",
+		config_inheritance
+	},
+
 	{
 		"LOG",
 		"",
@@ -264,6 +280,13 @@ struct config_type config_table[] =
 		config_packetpatch
 	},
 
+	{
+		"RANDOM SEED",
+		"",
+		"Seed value used for the random number engine",
+		config_randomseed
+	},
+
 	{
 		"REPEAT ENTER",
 		"You send the last command on an enter",
@@ -278,6 +301,13 @@ struct config_type config_table[] =
 		config_repeatchar
 	},
 
+	{
+		"SCREEN READER",
+		"You are using a screen reader",
+		"You are not using a screen reader",
+		config_screenreader
+	},
+
 	{
 		"SCROLL LOCK",
 		"You do not see mud output while scrolling",
@@ -327,12 +357,6 @@ struct config_type config_table[] =
 		config_wordwrap
 	},
 
-	{
-		"256 COLORS",
-		"Your terminal is 256 color capable",
-		"Your terminal is not 256 color capable",
-		config_256color
-	},
 
 	{
 		"",
@@ -458,6 +482,23 @@ struct chat_type chat_table[] =
 	{     "",                 NULL,                0, 0, ""                                               }
 };
 
+struct port_type port_table[] =
+{
+	{     "CALL",             port_call,           0, 0, "Create outgoing socket connection"              },
+	{     "COLOR",            port_color,          1, 0, "Set the default port message color"             },
+	{     "DND",              port_dnd,            0, 0, "Decline new connections"                        },
+	{     "GROUP",            port_group,          0, 1, "Assign a group to a socket"                     },
+	{     "IGNORE",           port_ignore,         1, 0, "Ignore all messages from a socket"              },
+	{     "INITIALIZE",       port_initialize,     0, 0, "Initialize port with optional file name"        },
+	{     "INFO",             port_info,           0, 0, "Display the port settings"                      },
+	{     "NAME",             port_name,           0, 0, "Change a socket name"                           },
+	{     "PREFIX",           port_prefix,         1, 0, "Prefix before each port message"                },
+	{     "SEND",             port_send,           0, 1, "Send a message to a socket"                     },
+	{     "UNINITIALIZE",     port_uninitialize,   0, 0, "Uninitializes the port"                         },
+	{     "WHO",              port_who,            0, 0, "Show all socket connections"                    },
+	{     "ZAP",              port_zap,            1, 0, "Close the connection to a socket"               },
+	{     "",                 NULL,                0, 0, ""                                               }
+};
 
 struct array_type array_table[] =
 {
@@ -472,6 +513,7 @@ struct array_type array_table[] =
 	{     "INSERT",           array_insert        },
 	{     "LENGTH",           array_size          },
 	{     "SET",              array_set           },
+	{     "SIMPLIFY",         array_simplify      },
 	{     "SIZE",             array_size          },
 	{     "SORT",             array_sort          },
 	{     "SRT",              array_sort          },
@@ -540,6 +582,21 @@ struct cursor_type cursor_table[] =
 		"",
 		cursor_backspace
 	},
+
+	{
+		"BRACE OPEN",
+		"Insert the { character",
+		"",
+		cursor_brace_open
+	},
+
+	{
+		"BRACE CLOSE",
+		"Insert the } character",
+		"",
+		cursor_brace_close
+	},
+
 	{
 		"BACKWARD",
 		"Move cursor backward",
@@ -660,6 +717,12 @@ struct cursor_type cursor_table[] =
 		"",
 		cursor_home
 	},
+	{
+		"INFO",
+		"Print debugging information",
+		"",
+		cursor_info
+	},
 	{
 		"INSERT",
 		"Turn insert mode on or off",
@@ -714,12 +777,6 @@ struct cursor_type cursor_table[] =
 		"",
 		cursor_suspend
 	},
-	{
-		"TEST",
-		"Print debugging information",
-		"",
-		cursor_test
-	},
 	{
 		"TAB BACKWARD",
 		"Tab completion from tab list, backward",
@@ -816,6 +873,7 @@ struct timer_type timer_table[] =
 	{    "Poll Stdin"                  },
 	{    "Poll Sessions"               },
 	{    "Poll Chat Server"            },
+	{    "Poll Port Sessions"          },
 	{    "Update Tickers"              },
 	{    "Update Delays"               },
 	{    "Update Packet Patcher"       },
@@ -840,6 +898,9 @@ struct event_type event_table[] =
 	{    "MAP EXIT ROOM",                          "Triggers when exiting a map room."       },
 	{    "MINUTE",                                 "Triggers each minute or given minute."   },
 	{    "MONTH",                                  "Triggers each month or given month."     },
+	{    "PORT CONNECTION",                        "Triggers when socket connects."          },
+	{    "PORT DISCONNECTION",                     "Triggers when socket disconnects."       },
+	{    "PORT MESSAGE",                           "Triggers when socket data is received."  },
 	{    "PROGRAM START",                          "Triggers when main session starts."      },
 	{    "PROGRAM TERMINATION",                    "Triggers when main session exists."      },
 	{    "RECEIVED INPUT",                         "Triggers when new input is received."    },
@@ -861,6 +922,7 @@ struct event_type event_table[] =
 	{    "VT100 DA",                               "Triggers on an ESC [ c call."            },
 	{    "VT100 DECID",                            "Triggers on an ESC Z call."              },
 	{    "VT100 DSR",                              "Triggers on an ESC [ 5 n call."          },
+	{    "VT100 ENQ",                              "Triggers on an \\x05 call."              },
 	{    "WEEK",                                   "Triggers each week or given week."       },
 	{    "YEAR",                                   "Triggers each year or given year."       },
 	{    "",                                       ""                                        }
@@ -892,6 +954,7 @@ struct line_type line_table[] =
 	{    "QUIET",             line_quiet             },
 	{    "STRIP",             line_strip             },
 	{    "SUBSTITUTE",        line_substitute        },
+	{    "VERBATIM",          line_verbatim          },
 	{    "VERBOSE",           line_verbose           },
 	{    "",                  NULL                   }
 };
