@@ -142,6 +142,30 @@ int translate_telopts(struct session *ses, unsigned char *src, int cplen)
 
 		switch (inflate(ses->mccp, Z_SYNC_FLUSH))
 		{
+			case Z_BUF_ERROR:
+				if (ses->mccp->avail_out == 0)
+				{
+					gtd->mccp_len *= 2;
+					gtd->mccp_buf  = (unsigned char *) realloc(gtd->mccp_buf, gtd->mccp_len);
+
+					ses->mccp->avail_out = gtd->mccp_len / 2;
+					ses->mccp->next_out  = gtd->mccp_buf + gtd->mccp_len / 2;
+
+					goto inflate;
+				}
+				else
+				{
+					tintin_puts2(ses, "");
+					tintin_puts2(ses, "#COMPRESSION ERROR, Z_BUF_ERROR, DISABLING MCCP.");
+					send_dont_mccp2(ses, 0, NULL);
+					inflateEnd(ses->mccp);
+					free(ses->mccp);
+					ses->mccp = NULL;
+					cpsrc = src;
+					cplen = 0;
+				}
+				break;
+
 			case Z_OK:
 				if (ses->mccp->avail_out == 0)
 				{
@@ -569,6 +593,14 @@ int mark_prompt(struct session *ses, int cplen, unsigned char *cpsrc)
 {
 	SET_BIT(ses->telopts, TELOPT_FLAG_PROMPT);
 
+	if (cpsrc[1] == GA)
+	{
+		check_all_events(ses, SUB_ARG, 0, 0, "IAC GA");
+	}
+	else if (cpsrc[1] == EOR)
+	{
+		check_all_events(ses, SUB_ARG, 0, 0, "IAC EOR");
+	}
 	return 2;
 }
 
@@ -595,7 +627,13 @@ int recv_sb_ttype(struct session *ses, int cplen, unsigned char *cpsrc)
 		{
 			char mtts[BUFFER_SIZE];
 
-			sprintf(mtts, "MTTS %d", 1 + (HAS_BIT(ses->flags, SES_FLAG_SPLIT) ? 0 : 2) + (HAS_BIT(ses->flags, SES_FLAG_UTF8) ? 4 : 0) + (HAS_BIT(ses->flags, SES_FLAG_256COLOR) ? 8 : 0));
+			sprintf(mtts, "MTTS %d",
+				(HAS_BIT(ses->flags, SES_FLAG_ANSICOLOR) ? 1 : 0) +
+				(HAS_BIT(ses->flags, SES_FLAG_SPLIT) ? 0 : 2) +
+				(HAS_BIT(ses->flags, SES_FLAG_UTF8) ? 4 : 0) +
+				(HAS_BIT(ses->flags, SES_FLAG_256COLOR) ? 8 : 0) +
+				(HAS_BIT(ses->flags, SES_FLAG_SCREENREADER) ? 64 : 0) +
+				(HAS_BIT(ses->flags, SES_FLAG_TRUECOLOR) ? 256 : 0));
 
 			socket_printf(ses, 6 + strlen(mtts), "%c%c%c%c%s%c%c", IAC, SB, TELOPT_TTYPE, 0, mtts, IAC, SE);
 
