@@ -36,7 +36,7 @@ int match(struct session *ses, char *str, char *exp, int flags)
 {
 	char expbuf[BUFFER_SIZE];
 
-	sprintf(expbuf, "^%s$", exp);
+	sprintf(expbuf, "\\A%s\\Z", exp);
 
 	substitute(ses, expbuf, expbuf, flags);
 
@@ -177,17 +177,100 @@ pcre *regexp_compile(char *exp, int option)
 }
 
 
-/******************************************************************************
-* copy *string into *result, but substitute the various expressions with the  *
-* values they stand for.                                                      *
-******************************************************************************/
+int is_variable(struct session *ses, char *str)
+{
+	struct listroot *root;
+	char temp[BUFFER_SIZE], *ptt;
+	int i = 1;
+
+	while (str[i] == str[0])
+	{
+		i++;
+	}
+
+	if (str[i] == DEFAULT_OPEN)
+	{
+		return TRUE;
+	}
+
+	if (str[i] != '_' && isalpha((int) str[i]) == 0)
+	{
+		return FALSE;
+	}
+
+	ptt = temp;
+
+	while (isalnum((int) str[i]) || str[i] == '_')
+	{
+		*ptt++ = str[i];
+
+		i++;
+	}
+	*ptt = 0;
+
+	root = local_list(ses);
+
+	if (search_node_list(root, temp) == NULL)
+	{
+		root = ses->list[LIST_VARIABLE];
+
+		if (search_node_list(root, temp) == NULL)
+		{
+			return FALSE;
+		}
+	}
+
+	return TRUE;
+}
+
+int is_function(struct session *ses, char *str)
+{
+	char temp[BUFFER_SIZE], *ptt;
+	int i = 1;
+
+	while (str[i] == str[0])
+	{
+		i++;
+	}
+
+	if (str[i] != '_' && isalpha((int) str[i]) == 0)
+	{
+		return FALSE;
+	}
+
+	ptt = temp;
+
+	while (isalnum((int) str[i]) || str[i] == '_')
+	{
+		*ptt++ = str[i];
+
+		i++;
+	}
+	*ptt = 0;
+
+	if (str[i] != DEFAULT_OPEN)
+	{
+		return FALSE;
+	}
+
+	if (search_node_list(ses->list[LIST_FUNCTION], temp) == NULL)
+	{
+		if (find_session(temp) == NULL)
+		{
+			return FALSE;
+		}
+	}
+
+	return TRUE;
+}
 
 int substitute(struct session *ses, char *string, char *result, int flags)
 {
 	struct listnode *node;
+	struct listroot *root;
 	struct session *sesptr;
 	char temp[BUFFER_SIZE], buf[BUFFER_SIZE], buffer[BUFFER_SIZE], *pti, *pto, *ptt, *str;
-	char *pte, old[6] = { 0 };
+	char *pte, old[10] = { 0 };
 	int i, cnt, escape = FALSE, flags_neol = flags;
 
 	push_call("substitute(%p,%p,%p,%d)",ses,string,result,flags);
@@ -263,6 +346,15 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 					}
 					*ptt = 0;
 
+					if (pti[i] != DEFAULT_OPEN)
+					{
+						while (*pti == '@')
+						{
+							*pto++ = *pti++;
+						}
+						continue;
+					}
+
 					node = search_node_list(ses->list[LIST_FUNCTION], temp);
 
 					if (node == NULL)
@@ -270,7 +362,7 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 						sesptr = find_session(temp);
 					}
 
-					if ((sesptr == NULL && node == NULL) || pti[i] != DEFAULT_OPEN)
+					if (sesptr == NULL && node == NULL)
 					{
 						while (*pti == '@')
 						{
@@ -319,6 +411,13 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 
 						if (*pte == 0)
 						{
+							while (++i < 100)
+							{
+								if (*gtd->vars[i])
+								{
+									RESTRING(gtd->vars[i], "");
+								}
+							}
 							break;
 						}
 
@@ -339,18 +438,22 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 				}
 				else
 				{
+					if (HAS_BIT(flags, SUB_SEC) && !HAS_BIT(flags, SUB_ARG) && is_function(ses, pti))
+					{
+						*pto++ = '\\';
+					}
 					*pto++ = *pti++;
 				}
 				break;
 
-			case '$':
-				if (HAS_BIT(flags, SUB_VAR) && !HAS_BIT(ses->list[LIST_VARIABLE]->flags, LIST_FLAG_IGNORE))
+			case '*':
+				if (HAS_BIT(flags, SUB_VAR) && !HAS_BIT(ses->list[LIST_VARIABLE]->flags, LIST_FLAG_IGNORE) && pti[1])
 				{
-					int def = FALSE;
+					int brace = FALSE;
 					i = 1;
 					escape = FALSE;
 
-					while (pti[i] == '$')
+					while (pti[i] == '*')
 					{
 						escape = TRUE;
 
@@ -359,7 +462,7 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 
 					if (pti[i] == DEFAULT_OPEN)
 					{
-						def = TRUE;
+						brace = TRUE;
 
 						ptt = get_arg_in_braces(ses, &pti[i], buf, TRUE);
 
@@ -380,9 +483,122 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 						*ptt = 0;
 					}
 
-					node = search_node_list(ses->list[LIST_VARIABLE], temp);
+					if (*temp)
+					{
+						root = local_list(ses);
 
-					if (def == FALSE && node == NULL)
+						if ((node = search_node_list(root, temp)) == NULL)
+						{
+							root = ses->list[LIST_VARIABLE];
+
+							node = search_node_list(root, temp);
+						}
+					}
+					else
+					{
+						root = ses->list[LIST_VARIABLE];
+						node = NULL;
+					}
+
+					if (brace == FALSE && node == NULL)
+					{
+						while (*pti == '*')
+						{
+							*pto++ = *pti++;
+						}
+						continue;
+					}
+
+					if (escape)
+					{
+						pti++;
+
+						while (*pti == '*')
+						{
+							*pto++ = *pti++;
+						}
+						continue;
+					}
+
+					pti = get_arg_at_brackets(ses, &pti[i], temp + strlen(temp));
+
+					substitute(ses, temp, buf, flags_neol);
+
+					str = str_dup("");
+
+					get_nest_node_key(root, buf, &str, brace);
+
+					substitute(ses, str, pto, flags_neol - SUB_VAR);
+
+					pto += strlen(pto);
+
+					str_free(str);
+				}
+				else
+				{
+					if (HAS_BIT(flags, SUB_SEC) && !HAS_BIT(flags, SUB_ARG) && is_variable(ses, pti))
+					{
+						*pto++ = '\\';
+					}
+					*pto++ = *pti++;
+				}
+				break;
+
+			case '$':
+				if (HAS_BIT(flags, SUB_VAR) && !HAS_BIT(ses->list[LIST_VARIABLE]->flags, LIST_FLAG_IGNORE) && pti[1])
+				{
+					int brace = FALSE;
+					i = 1;
+					escape = FALSE;
+
+					while (pti[i] == '$')
+					{
+						escape = TRUE;
+
+						i++;
+					}
+
+					if (pti[i] == DEFAULT_OPEN)
+					{
+						brace = TRUE;
+
+						ptt = get_arg_in_braces(ses, &pti[i], buf, TRUE);
+
+						i += strlen(buf) + 2;
+
+						substitute(ses, buf, temp, flags_neol);
+					}
+					else
+					{
+						ptt = temp;
+
+						while (isalnum((int) pti[i]) || pti[i] == '_')
+						{
+							*ptt++ = pti[i];
+
+							i++;
+						}
+						*ptt = 0;
+					}
+
+					if (*temp)
+					{
+						root = local_list(ses);
+
+						if ((node = search_node_list(root, temp)) == NULL)
+						{
+							root = ses->list[LIST_VARIABLE];
+
+							node = search_node_list(root, temp);
+						}
+					}
+					else
+					{
+						root = ses->list[LIST_VARIABLE];
+						node = NULL;
+					}
+
+					if (brace == FALSE && node == NULL)
 					{
 						while (*pti == '$')
 						{
@@ -408,7 +624,7 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 
 					str = str_dup("");
 
-					get_nest_node(ses->list[LIST_VARIABLE], buf, &str, def);
+					get_nest_node_val(root, buf, &str, brace);
 
 					substitute(ses, str, pto, flags_neol - SUB_VAR);
 
@@ -418,6 +634,10 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 				}
 				else
 				{
+					if (HAS_BIT(flags, SUB_SEC) && !HAS_BIT(flags, SUB_ARG) && is_variable(ses, pti))
+					{
+						*pto++ = '\\';
+					}
 					*pto++ = *pti++;
 				}
 				break;
@@ -453,7 +673,7 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 				}
 				else if (HAS_BIT(flags, SUB_VAR) && !HAS_BIT(ses->list[LIST_VARIABLE]->flags, LIST_FLAG_IGNORE))
 				{
-					int def = FALSE;
+					int brace = FALSE;
 					i = 1;
 					escape = FALSE;
 
@@ -466,7 +686,7 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 
 					if (pti[i] == DEFAULT_OPEN)
 					{
-						def = TRUE;
+						brace = TRUE;
 
 						ptt = get_arg_in_braces(ses, &pti[i], buf, TRUE);
 
@@ -487,9 +707,23 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 						*ptt = 0;
 					}
 
-					node = search_node_list(ses->list[LIST_VARIABLE], temp);
+					if (*temp)
+					{
+						root = local_list(ses);
 
-					if (def == FALSE && node == NULL)
+						if ((node = search_node_list(root, temp)) == NULL)
+						{
+							root = ses->list[LIST_VARIABLE];
+							node = search_node_list(root, temp);
+						}
+					}
+					else
+					{
+						root = ses->list[LIST_VARIABLE];
+						node = NULL;
+					}
+
+					if (brace == FALSE && node == NULL)
 					{
 						while (*pti == '&')
 						{
@@ -515,7 +749,7 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 
 					str = str_dup("");
 
-					get_nest_index(ses->list[LIST_VARIABLE], buf, &str, def);
+					get_nest_index(root, buf, &str, brace);
 
 					substitute(ses, str, pto, flags_neol - SUB_VAR);
 
@@ -525,6 +759,10 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 				}
 				else
 				{
+					if (HAS_BIT(flags, SUB_SEC) && !HAS_BIT(flags, SUB_ARG) && is_variable(ses, pti))
+					{
+						*pto++ = '\\';
+					}
 					*pto++ = *pti++;
 				}
 				break;
@@ -579,10 +817,29 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 										break;
 
 									case '$':
-									case '@':
 									case '&':
-										*pto++ = '\\';
-										*pto++ = *ptt;
+									case '*':
+										if (is_variable(ses, ptt))
+										{
+											*pto++ = '\\';
+											*pto++ = *ptt;
+										}
+										else
+										{
+											*pto++ = *ptt;
+										}
+										break;
+
+									case '@':
+										if (is_function(ses, ptt))
+										{
+											*pto++ = '\\';
+											*pto++ = *ptt;
+										}
+										else
+										{
+											*pto++ = *ptt;
+										}
 										break;
 
 									case COMMAND_SEPARATOR:
@@ -606,16 +863,30 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 				}
 				else
 				{
-					*pto++ = *pti++;
+					if (HAS_BIT(flags, SUB_VAR) && pti[1] == '*')
+					{
+						*pto++ = *pti++;
+						*pto++ = *pti++;
+					}
+					if (HAS_BIT(flags, SUB_VAR) && pti[1] == '!' && pti[2] == '*')
+					{
+						*pto++ = *pti++;
+						*pto++ = *pti++;
+						*pto++ = *pti++;
+					}
+					else
+					{
+						*pto++ = *pti++;
+					}
 				}
 				break;
 
 			case '<':
-				if (HAS_BIT(flags, SUB_COL))
+				if (HAS_BIT(flags, SUB_COL) && isxdigit((int) pti[1]))
 				{
-					if (HAS_BIT(flags, SUB_CMP) && !strncmp(old, pti, 5))
+					if (HAS_BIT(flags, SUB_CMP) && old[0] && !strncmp(old, pti, strlen(old)))
 					{
-						pti += 5;
+						pti += strlen(old);
 					}
 					else if (isdigit((int) pti[1]) && isdigit((int) pti[2]) && isdigit((int) pti[3]) && pti[4] == '>')
 					{
@@ -708,7 +979,7 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 						*pto++ = '0' + cnt % 100 / 10;
 						*pto++ = '0' + cnt % 10;
 						*pto++ = 'm';
-						pti += sprintf(old, "<%c%c%c>", pti[1], pti[2], pti[3]);
+						pti += sprintf(old, "<g%c%c>", pti[2], pti[3]);
 					}
 					else if (pti[1] == 'G' && isdigit((int) pti[2]) && isdigit((int) pti[3]) && pti[4] == '>')
 					{
@@ -724,7 +995,123 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 						*pto++ = '0' + cnt % 100 / 10;
 						*pto++ = '0' + cnt % 10;
 						*pto++ = 'm';
-						pti += sprintf(old, "<%c%c%c>", pti[1], pti[2], pti[3]);
+						pti += sprintf(old, "<G%c%c>", pti[2], pti[3]);
+					}
+					else if (toupper((int) pti[1]) == 'F' && isxdigit((int) pti[2]) && isxdigit((int) pti[3]) && isxdigit((int) pti[3]) && pti[5] == '>')
+					{
+						*pto++ = ESCAPE;
+						*pto++ = '[';
+						*pto++ = '3';
+						*pto++ = '8';
+						*pto++ = ';';
+						*pto++ = '2';
+						*pto++ = ';';
+						cnt  = isdigit(pti[2]) ? (pti[2] - '0') : (pti[2] - 'A' + 10);
+						cnt += cnt * 16;
+						*pto++ = '0' + cnt / 100;
+						*pto++ = '0' + cnt % 100 / 10;
+						*pto++ = '0' + cnt % 10;
+						*pto++ = ';';
+						cnt  = isdigit(pti[3]) ? (pti[3] - '0') : (pti[3] - 'A' + 10);
+						cnt += cnt * 16;
+						*pto++ = '0' + cnt / 100;
+						*pto++ = '0' + cnt % 100 / 10;
+						*pto++ = '0' + cnt % 10;
+						*pto++ = ';';
+						cnt  = isdigit(pti[4]) ? (pti[4] - '0') : (pti[4] - 'A' + 10);
+						cnt += cnt * 16;
+						*pto++ = '0' + cnt / 100;
+						*pto++ = '0' + cnt % 100 / 10;
+						*pto++ = '0' + cnt % 10;
+						*pto++ = 'm';
+						pti += sprintf(old, "<F%c%c%c>", pti[2], pti[3], pti[4]);
+					}
+					else if (toupper((int) pti[1]) == 'F' && isxdigit((int) pti[2]) && isxdigit((int) pti[3]) && isxdigit((int) pti[4]) && isxdigit((int) pti[5]) && isxdigit((int) pti[6]) && isxdigit((int) pti[7]) && pti[8] == '>')
+					{
+						*pto++ = ESCAPE;
+						*pto++ = '[';
+						*pto++ = '3';
+						*pto++ = '8';
+						*pto++ = ';';
+						*pto++ = '2';
+						*pto++ = ';';
+						cnt  = isdigit(pti[2]) ? 16 * (pti[2] - '0') : 16 * (pti[2] - 'A' + 10);
+						cnt += isdigit(pti[3]) ?  1 * (pti[3] - '0') :  1 * (pti[3] - 'A' + 10);
+						*pto++ = '0' + cnt / 100;
+						*pto++ = '0' + cnt % 100 / 10;
+						*pto++ = '0' + cnt % 10;
+						*pto++ = ';';
+						cnt  = isdigit(pti[4]) ? 16 * (pti[4] - '0') : 16 * (pti[4] - 'A' + 10);
+						cnt += isdigit(pti[5]) ?  1 * (pti[5] - '0') :  1 * (pti[5] - 'A' + 10);
+						*pto++ = '0' + cnt / 100;
+						*pto++ = '0' + cnt % 100 / 10;
+						*pto++ = '0' + cnt % 10;
+						*pto++ = ';';
+						cnt  = isdigit(pti[6]) ? 16 * (pti[6] - '0') : 16 * (pti[6] - 'A' + 10);
+						cnt += isdigit(pti[7]) ?  1 * (pti[7] - '0') :  1 * (pti[7] - 'A' + 10);
+						*pto++ = '0' + cnt / 100;
+						*pto++ = '0' + cnt % 100 / 10;
+						*pto++ = '0' + cnt % 10;
+						*pto++ = 'm';
+						pti += sprintf(old, "<F%c%c%c%c%c%c>", pti[2], pti[3], pti[4], pti[5], pti[6], pti[7]);
+					}
+					else if (toupper((int) pti[1]) == 'B' && isxdigit((int) pti[2]) && isxdigit((int) pti[3]) && isxdigit((int) pti[3]) && pti[5] == '>')
+					{
+						*pto++ = ESCAPE;
+						*pto++ = '[';
+						*pto++ = '4';
+						*pto++ = '8';
+						*pto++ = ';';
+						*pto++ = '2';
+						*pto++ = ';';
+						cnt  = isdigit(pti[2]) ? (pti[2] - '0') : (pti[2] - 'A' + 10);
+						cnt += cnt * 16;
+						*pto++ = '0' + cnt / 100;
+						*pto++ = '0' + cnt % 100 / 10;
+						*pto++ = '0' + cnt % 10;
+						*pto++ = ';';
+						cnt  = isdigit(pti[3]) ? (pti[3] - '0') : (pti[3] - 'A' + 10);
+						cnt += cnt * 16;
+						*pto++ = '0' + cnt / 100;
+						*pto++ = '0' + cnt % 100 / 10;
+						*pto++ = '0' + cnt % 10;
+						*pto++ = ';';
+						cnt  = isdigit(pti[4]) ? (pti[4] - '0') : (pti[4] - 'A' + 10);
+						cnt += cnt * 16;
+						*pto++ = '0' + cnt / 100;
+						*pto++ = '0' + cnt % 100 / 10;
+						*pto++ = '0' + cnt % 10;
+						*pto++ = 'm';
+						pti += sprintf(old, "<B%c%c%c>", pti[2], pti[3], pti[4]);
+					}
+					else if (toupper((int) pti[1]) == 'B' && isxdigit((int) pti[2]) && isxdigit((int) pti[3]) && isxdigit((int) pti[4]) && isxdigit((int) pti[5]) && isxdigit((int) pti[6]) && isxdigit((int) pti[7]) && pti[8] == '>')
+					{
+						*pto++ = ESCAPE;
+						*pto++ = '[';
+						*pto++ = '4';
+						*pto++ = '8';
+						*pto++ = ';';
+						*pto++ = '2';
+						*pto++ = ';';
+						cnt  = isdigit(pti[2]) ? 16 * (pti[2] - '0') : 16 * (pti[2] - 'A' + 10);
+						cnt += isdigit(pti[3]) ?  1 * (pti[3] - '0') :  1 * (pti[3] - 'A' + 10);
+						*pto++ = '0' + cnt / 100;
+						*pto++ = '0' + cnt % 100 / 10;
+						*pto++ = '0' + cnt % 10;
+						*pto++ = ';';
+						cnt  = isdigit(pti[4]) ? 16 * (pti[4] - '0') : 16 * (pti[4] - 'A' + 10);
+						cnt += isdigit(pti[5]) ?  1 * (pti[5] - '0') :  1 * (pti[5] - 'A' + 10);
+						*pto++ = '0' + cnt / 100;
+						*pto++ = '0' + cnt % 100 / 10;
+						*pto++ = '0' + cnt % 10;
+						*pto++ = ';';
+						cnt  = isdigit(pti[6]) ? 16 * (pti[6] - '0') : 16 * (pti[6] - 'A' + 10);
+						cnt += isdigit(pti[7]) ?  1 * (pti[7] - '0') :  1 * (pti[7] - 'A' + 10);
+						*pto++ = '0' + cnt / 100;
+						*pto++ = '0' + cnt % 100 / 10;
+						*pto++ = '0' + cnt % 10;
+						*pto++ = 'm';
+						pti += sprintf(old, "<B%c%c%c%c%c%c>", pti[2], pti[3], pti[4], pti[5], pti[6], pti[7]);
 					}
 					else
 					{
@@ -759,7 +1146,10 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 							}
 							break;
 						case 'e':
-							*pto++ = '\033';
+							*pto++ = '\e';
+							break;
+						case 'f':
+							*pto++ = '\f';
 							break;
 						case 'n':
 							*pto++ = '\n';
@@ -773,13 +1163,46 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 						case 'x':
 							if (pti[1] && pti[2])
 							{
-								pti++;
-								*pto++ = hex_number(pti);
-								pti++;
+								if (pti[1] == '0' && pti[2] == '0' && pti[3] == 0)
+								{
+									pti += 2;
+									DEL_BIT(flags, SUB_EOL);
+									DEL_BIT(flags, SUB_LNF);
+								}
+								else
+								{
+									pti++;
+									*pto++ = hex_number(pti);
+									pti++;
+								}
 							}
 							break;
+
+						case 'u':
+							if (pti[1] && pti[2] && pti[3] && pti[4])
+							{
+								pto += unicode_16_bit(&pti[1], pto);
+								pti += 4;
+							}
+							break;
+						case 'U':
+							if (pti[1] && pti[2] && pti[3] && pti[4] && pti[5] && pti[6])
+							{
+								pto += unicode_21_bit(&pti[1], pto);
+								pti += 6;
+							}
+							break;
+
+						case 'v':
+							*pto++ = '\v';
+							break;
 						case '0':
-							if (pti[1] && pti[2])
+							if (pti[1] == 0)
+							{
+								DEL_BIT(flags, SUB_EOL);
+								DEL_BIT(flags, SUB_LNF);
+							}
+							else if (pti[1] && pti[2])
 							{
 								pti++;
 								*pto++ = oct_number(pti);
@@ -796,6 +1219,11 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 					}
 					pti++;
 				}
+				else if (HAS_BIT(flags, SUB_SEC) && !HAS_BIT(flags, SUB_ARG))
+				{
+					*pto++ = '\\';
+					*pto++ = *pti++;
+				}
 				else
 				{
 					*pto++ = *pti++;
@@ -808,11 +1236,6 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 				{
 					switch (*pti)
 					{
-						case '\\':
-							*pto++ = '\\';
-							*pto++ = '\\';
-							break;
-
 						case '{':
 							*pto++ = '\\';
 							*pto++ = 'x';
@@ -827,13 +1250,6 @@ int substitute(struct session *ses, char *string, char *result, int flags)
 							*pto++ = 'D';
 							break;
 
-						case '$':
-						case '@':
-						case '&':
-							*pto++ = '\\';
-							*pto++ = *pti;
-							break;
-
 						case COMMAND_SEPARATOR:
 							*pto++ = '\\';
 							*pto++ = COMMAND_SEPARATOR;
@@ -950,6 +1366,24 @@ int tintin_regexp_check(struct session *ses, char *exp)
 					case '.':
 					case '%':
 						return TRUE;
+
+					case '!':
+						switch (exp[2])
+						{
+							case 'd':
+							case 'D':
+							case 's':
+							case 'S':
+							case 'w':
+							case 'W':
+							case '?':
+							case '*':
+							case '+':
+							case '.':
+							case '{':
+								return TRUE;
+						}
+						break;
 				}
 				break;
 		}
@@ -976,6 +1410,24 @@ int tintin_regexp(struct session *ses, pcre *nodepcre, char *str, char *exp, int
 		if (HAS_BIT(ses->flags, SES_FLAG_BIG5) && *pti & 128 && pti[1] != 0)
 		{
 			*pto++ = *pti++;
+
+			switch (*pti)
+			{
+				case '\\':
+				case '[':
+				case ']':
+				case '(':
+				case ')':
+				case '|':
+				case '.':
+				case '?':
+				case '+':
+				case '*':
+				case '$':
+				case '^':
+					*pto++ = '\\';
+					break;
+			}
 			*pto++ = *pti++;
 			continue;
 		}
@@ -1135,6 +1587,80 @@ int tintin_regexp(struct session *ses, pcre *nodepcre, char *str, char *exp, int
 						pti++;
 						break;
 
+					case '!':
+						switch (pti[2])
+						{
+							case 'd':
+								pti += 3;
+								strcpy(pto, *pti == 0 ? "[0-9]*" : "[0-9]*?");
+								pto += strlen(pto);
+								break;
+
+							case 'D':
+								pti += 3;
+								strcpy(pto, *pti == 0 ? "[^0-9]*" : "[^0-9]*?");
+									pto += strlen(pto);
+								break;
+
+							case 's':
+								pti += 3;
+								strcpy(pto, *pti == 0 ? "\\s*" : "\\s*?");
+								pto += strlen(pto);
+								break;
+
+							case 'S':
+								pti += 3;
+								strcpy(pto, *pti == 0 ? "\\S*" : "\\S*?");
+								pto += strlen(pto);
+								break;
+
+							case 'w':
+								pti += 3;
+								strcpy(pto, *pti == 0 ? "[a-zA-Z]*" : "[a-zA-Z]*?");
+								pto += strlen(pto);
+								break;
+
+							case 'W':
+								pti += 3;
+								strcpy(pto, *pti == 0 ? "[^a-zA-Z]*" : "[^a-zA-Z]*?");
+								pto += strlen(pto);
+								break;
+
+							case '?':
+								pti += 3;
+								strcpy(pto, *pti == 0 ? ".?" : ".?" "?");
+								pto += strlen(pto);
+								break;
+
+							case '*':
+								pti += 3;
+								strcpy(pto, *pti == 0 ? ".*" : ".*?");
+								pto += strlen(pto);
+								break;
+
+							case '+':
+								pti += 3;
+								strcpy(pto, *pti == 0 ? ".+" : ".+?");
+								pto += strlen(pto);
+								break;
+
+							case '.':
+								pti += 3;
+								strcpy(pto, ".");
+								pto += strlen(pto);
+								break;
+
+							case '{':
+								pti = get_arg_in_braces(ses, pti+2, pto, TRUE);
+								pto += strlen(pto);
+								break;
+
+							default:
+								*pto++ = *pti++;
+								break;
+						}
+						break;
+
 					default:
 						*pto++ = *pti++;
 						break;
@@ -1177,6 +1703,24 @@ pcre *tintin_regexp_compile(struct session *ses, struct listnode *node, char *ex
 		if (HAS_BIT(ses->flags, SES_FLAG_BIG5) && *pti & 128 && pti[1] != 0)
 		{
 			*pto++ = *pti++;
+
+			switch (*pti)
+			{
+				case '\\':
+				case '[':
+				case ']':
+				case '(':
+				case ')':
+				case '|':
+				case '.':
+				case '?':
+				case '+':
+				case '*':
+				case '$':
+				case '^':
+					*pto++ = '\\';
+					break;
+			}
 			*pto++ = *pti++;
 			continue;
 		}
@@ -1350,6 +1894,91 @@ pcre *tintin_regexp_compile(struct session *ses, struct listnode *node, char *ex
 						pti++;
 						break;
 
+					case '!':
+						switch (pti[2])
+						{
+							case 'd':
+								pti += 3;
+								strcpy(pto, *pti == 0 ? "[0-9]*" : "[0-9]*?");
+								pto += strlen(pto);
+								break;
+
+							case 'D':
+								pti += 3;
+								strcpy(pto, *pti == 0 ? "[^0-9]*" : "[^0-9]*?");
+									pto += strlen(pto);
+								break;
+
+							case 's':
+								pti += 3;
+								strcpy(pto, *pti == 0 ? "\\s*" : "\\s*?");
+								pto += strlen(pto);
+								break;
+
+							case 'S':
+								pti += 3;
+								strcpy(pto, *pti == 0 ? "\\S*" : "\\S*?");
+								pto += strlen(pto);
+								break;
+
+							case 'w':
+								pti += 3;
+								strcpy(pto, *pti == 0 ? "[a-zA-Z]*" : "[a-zA-Z]*?");
+								pto += strlen(pto);
+								break;
+
+							case 'W':
+								pti += 3;
+								strcpy(pto, *pti == 0 ? "[^a-zA-Z]*" : "[^a-zA-Z]*?");
+								pto += strlen(pto);
+								break;
+
+							case '?':
+								pti += 3;
+								strcpy(pto, *pti == 0 ? ".?" : ".?" "?");
+								pto += strlen(pto);
+								break;
+
+							case '*':
+								pti += 3;
+								strcpy(pto, *pti == 0 ? ".*" : ".*?");
+								pto += strlen(pto);
+								break;
+
+							case '+':
+								pti += 3;
+								strcpy(pto, *pti == 0 ? ".+" : ".+?");
+								pto += strlen(pto);
+								break;
+
+							case '.':
+								pti += 3;
+								strcpy(pto, ".");
+								pto += strlen(pto);
+								break;
+
+							case '{':
+								pti = get_arg_in_braces(ses, pti+2, pto, TRUE);
+
+								while (*pto)
+								{
+									if (pto[0] == '$' || pto[0] == '@')
+									{
+										if (pto[1])
+										{
+											return NULL;
+										}
+									}
+									pto++;
+								}
+								break;
+
+							default:
+								*pto++ = *pti++;
+								break;
+						}
+						break;
+
 					default:
 						*pto++ = *pti++;
 						break;
@@ -1363,5 +1992,7 @@ pcre *tintin_regexp_compile(struct session *ses, struct listnode *node, char *ex
 	}
 	*pto = 0;
 
+//	printf("debug regex compile (%s)\n", out);
+
 	return regexp_compile(out, option);
 }
