@@ -134,28 +134,30 @@
 #define TINTIN_DIR               ".tintin"
 #define HISTORY_FILE         "history.txt"
 
-#define STRING_SIZE                  65536
-#define BUFFER_SIZE                  32768
+#define STRING_SIZE                  64000
+#define BUFFER_SIZE                  32000
 #define NUMBER_SIZE                    100
+#define LEGEND_SIZE                     50
 #define LIST_SIZE                        2
 
 #define CLIENT_NAME              "TinTin++"
-#define CLIENT_VERSION           "2.01.1  "
+#define CLIENT_VERSION           "2.01.5  "
 
 #define ESCAPE                          27
 
 #define TIMER_POLL_INPUT                 0
 #define TIMER_POLL_SESSIONS              1
 #define TIMER_POLL_CHAT                  2
-#define TIMER_UPDATE_TICKS               3
-#define TIMER_UPDATE_DELAYS              4
-#define TIMER_UPDATE_PACKETS             5
-#define TIMER_UPDATE_CHAT                6
-#define TIMER_UPDATE_TERMINAL            7
-#define TIMER_UPDATE_TIME                8
-#define TIMER_UPDATE_MEMORY              9
-#define TIMER_STALL_PROGRAM             10
-#define TIMER_CPU                       11
+#define TIMER_POLL_PORT                  3
+#define TIMER_UPDATE_TICKS               4
+#define TIMER_UPDATE_DELAYS              5
+#define TIMER_UPDATE_PACKETS             6
+#define TIMER_UPDATE_CHAT                7
+#define TIMER_UPDATE_TERMINAL            8
+#define TIMER_UPDATE_TIME                9
+#define TIMER_UPDATE_MEMORY             10
+#define TIMER_STALL_PROGRAM             11
+#define TIMER_CPU                       12
 
 
 #define PULSE_PER_SECOND                20
@@ -163,13 +165,14 @@
 #define PULSE_POLL_INPUT                 1
 #define PULSE_POLL_SESSIONS              1
 #define PULSE_POLL_CHAT                  2
+#define PULSE_POLL_PORT                  2
 #define PULSE_UPDATE_TICKS               1
 #define PULSE_UPDATE_DELAYS              1
 #define PULSE_UPDATE_PACKETS             2
 #define PULSE_UPDATE_CHAT                2
 #define PULSE_UPDATE_TERMINAL            1
 #define PULSE_UPDATE_MEMORY              2
-#define PULSE_UPDATE_TIME               20
+#define PULSE_UPDATE_TIME                2
 
 /*
 	Index for lists used by tintin
@@ -231,13 +234,62 @@ enum operators
 	Various flags
 */
 
-#define COL_BLD                       (1 << 1)
-#define COL_UND                       (1 << 2)
-#define COL_BLK                       (1 << 3)
-#define COL_REV                       (1 << 4)
-#define COL_XTF                       (1 << 5)
-#define COL_XTB                       (1 << 6)
-#define COL_256                       (1 << 7)
+#define BV01 (1   <<  0)
+#define BV02 (1   <<  1)
+#define BV03 (1   <<  2)
+#define BV04 (1   <<  3)
+#define BV05 (1   <<  4)
+#define BV06 (1   <<  5)
+#define BV07 (1   <<  6)
+#define BV08 (1   <<  7)
+#define BV09 (1   <<  8)
+#define BV10 (1   <<  9)
+#define BV11 (1   << 10)
+#define BV12 (1   << 11)
+#define BV13 (1   << 12)
+#define BV14 (1   << 13)
+#define BV15 (1   << 14)
+#define BV16 (1   << 15)
+#define BV17 (1   << 16)
+#define BV18 (1   << 17)
+#define BV19 (1   << 18)
+#define BV20 (1   << 19)
+#define BV21 (1   << 20)
+#define BV22 (1   << 21)
+#define BV23 (1   << 22)
+#define BV24 (1   << 23)
+#define BV25 (1   << 24)
+#define BV26 (1   << 25)
+#define BV27 (1   << 26)
+#define BV28 (1   << 27)
+#define BV29 (1   << 28)
+#define BV30 (1   << 29)
+#define BV31 (1   << 30)
+#define BV32 (1LL << 31)
+#define BV33 (1LL << 32)
+#define BV34 (1LL << 33)
+#define BV35 (1LL << 34)
+#define BV36 (1LL << 35)
+#define BV37 (1LL << 36)
+#define BV38 (1LL << 37)
+#define BV39 (1LL << 38)
+
+#define COL_BLD                       BV01
+#define COL_UND                       BV02
+#define COL_BLK                       BV03
+#define COL_REV                       BV04
+#define COL_XTF                       BV05
+#define COL_XTF_5                     BV06
+#define COL_XTF_R                     BV07
+#define COL_XTB                       BV08
+#define COL_XTB_5                     BV09
+#define COL_XTB_R                     BV10
+#define COL_TCF                       BV11
+#define COL_TCF_2                     BV12
+#define COL_TCF_R                     BV13
+#define COL_TCB                       BV14
+#define COL_TCB_2                     BV15
+#define COL_TCB_R                     BV16
 
 #define CHAT_NAME_CHANGE                 1
 #define CHAT_REQUEST_CONNECTIONS         2
@@ -304,40 +356,46 @@ enum operators
 #define TINTIN_FLAG_HISTORYBROWSE     (1 <<  2)
 #define TINTIN_FLAG_HISTORYSEARCH     (1 <<  3)
 #define TINTIN_FLAG_PROCESSINPUT      (1 <<  4)
-#define TINTIN_FLAG_USERCOMMAND       (1 <<  5) /* Unused */
+#define TINTIN_FLAG_INHERITANCE       (1 <<  5)
 #define TINTIN_FLAG_INSERTINPUT       (1 <<  6)
 #define TINTIN_FLAG_VERBATIM          (1 <<  7) /* Unused */
 #define TINTIN_FLAG_TERMINATE         (1 <<  8)
 
-#define SES_FLAG_ECHOCOMMAND          (1 <<  1)
-#define SES_FLAG_SNOOP                (1 <<  2)
-#define SES_FLAG_MCCP                 (1 <<  3)
-#define SES_FLAG_MAPPING              (1 <<  4)
-#define SES_FLAG_SPLIT                (1 <<  5)
-#define SES_FLAG_SPEEDWALK            (1 <<  6)
-#define SES_FLAG_READMUD              (1 <<  8)
-#define SES_FLAG_WORDWRAP             (1 <<  9)
-#define SES_FLAG_VERBATIM             (1 << 10)
-#define SES_FLAG_CONNECTED            (1 << 11)
-#define SES_FLAG_REPEATENTER          (1 << 12)
-#define SES_FLAG_VERBOSE              (1 << 13)
-//#define SES_FLAG_VERBOSELINE          (1 << 14)
-#define SES_FLAG_LOGLEVEL             (1 << 15)
-#define SES_FLAG_LOGPLAIN             (1 << 16)
-#define SES_FLAG_LOGHTML              (1 << 17)
-#define SES_FLAG_GAG                  (1 << 18)
-#define SES_FLAG_UPDATEVTMAP          (1 << 19)
-#define SES_FLAG_COLORPATCH           (1 << 20)
-#define SES_FLAG_SCROLLLOCK           (1 << 21)
-#define SES_FLAG_SCAN                 (1 << 22)
-#define SES_FLAG_SCROLLSTOP           (1 << 23)
-#define SES_FLAG_CONVERTMETA          (1 << 24)
-#define SES_FLAG_RUN                  (1 << 25)
-#define SES_FLAG_UTF8                 (1 << 26)
-#define SES_FLAG_BIG5                 (1 << 27)
-#define SES_FLAG_256COLOR             (1 << 28)
-#define SES_FLAG_IGNORELINE           (1 << 29)
-#define SES_FLAG_CLOSED               (1 << 30)
+#define SES_FLAG_ECHOCOMMAND          BV01
+#define SES_FLAG_SNOOP                BV02
+#define SES_FLAG_MCCP                 BV03
+#define SES_FLAG_MAPPING              BV04 // #path related
+#define SES_FLAG_SPLIT                BV05
+#define SES_FLAG_SPEEDWALK            BV06
+#define SES_FLAG_READMUD              BV08
+#define SES_FLAG_WORDWRAP             BV09
+#define SES_FLAG_VERBATIM             BV10
+#define SES_FLAG_CONNECTED            BV11
+#define SES_FLAG_REPEATENTER          BV12
+#define SES_FLAG_VERBOSE              BV13
+#define SES_FLAG_LOGNEXT              BV14
+#define SES_FLAG_LOGLEVEL             BV15
+#define SES_FLAG_LOGPLAIN             BV16
+#define SES_FLAG_LOGHTML              BV17
+#define SES_FLAG_GAG                  BV18
+#define SES_FLAG_UPDATEVTMAP          BV19
+#define SES_FLAG_COLORPATCH           BV20
+#define SES_FLAG_SCROLLLOCK           BV21
+#define SES_FLAG_SCROLLSTOP           BV22
+#define SES_FLAG_SCAN                 BV23
+#define SES_FLAG_SCANABORT            BV24
+#define SES_FLAG_CONVERTMETA          BV25
+#define SES_FLAG_RUN                  BV26
+#define SES_FLAG_BIG5                 BV27
+#define SES_FLAG_UTF8                 BV28
+#define SES_FLAG_IGNORELINE           BV39
+#define SES_FLAG_CLOSED               BV30
+#define SES_FLAG_PORT                 BV31
+#define SES_FLAG_SCREENREADER         BV32
+#define SES_FLAG_ANSICOLOR            BV33
+#define SES_FLAG_256COLOR             BV34
+#define SES_FLAG_TRUECOLOR            BV35
+
 
 #define TELOPT_FLAG_SGA               (1 <<  0)
 #define TELOPT_FLAG_ECHO              (1 <<  1)
@@ -378,7 +436,7 @@ enum operators
 
 #define MAP_FLAG_STATIC               (1 <<  0)
 #define MAP_FLAG_VTMAP                (1 <<  1)
-#define MAP_FLAG_VTGRAPHICS           (1 <<  2)
+//#define MAP_FLAG_VTGRAPHICS           (1 <<  2)
 #define MAP_FLAG_ASCIIGRAPHICS        (1 <<  3)
 #define MAP_FLAG_ASCIIVNUMS           (1 <<  4)
 #define MAP_FLAG_MUDFONT              (1 <<  5)
@@ -423,7 +481,7 @@ enum operators
 
 #define STR_HASH_FLAG_NOGREP          (1 <<  0)
 
-#define MAX_STR_HASH                  10000
+#define MAX_STR_HASH                  20000
 
 /*
 	Some macros to deal with double linked lists
@@ -546,7 +604,7 @@ enum operators
 
 #define SCROLL(ses)               ((ses)->cur_row == 0 || ((ses)->cur_row >= (ses)->top_row && (ses)->cur_row <= (ses)->bot_row) || (ses)->cur_row == (ses)->rows)
 
-#define VERBATIM(ses)             (gtd->input_level == 0 && HAS_BIT((ses)->flags, SES_FLAG_VERBATIM))
+#define VERBATIM(ses)             (gtd->verbatim_level || (gtd->input_level == 0 && HAS_BIT((ses)->flags, SES_FLAG_VERBATIM)))
 
 #define DO_ARRAY(array) struct session *array (struct session *ses, struct listnode *list, char *arg, char *var)
 #define DO_CHAT(chat) void chat (char *left, char *right)
@@ -555,6 +613,7 @@ enum operators
 #define DO_CONFIG(config) struct session *config (struct session *ses, char *arg, int index)
 #define DO_MAP(map) void map (struct session *ses, char *arg, char *arg1, char *arg2)
 #define DO_PATH(path) void path (struct session *ses, char *arg)
+#define DO_PORT(port) struct session *port (struct session *ses, char *left, char *right, char *arg)
 #define DO_LINE(line) struct session *line (struct session *ses, char *arg)
 #define DO_CURSOR(cursor) void cursor (struct session *ses, char *arg)
 #define DO_HISTORY(history) void history (struct session *ses, char *arg)
@@ -605,13 +664,16 @@ struct session
 	struct session        * prev;
 	struct map_data       * map;
 	z_stream              * mccp;
+	struct port_data      * port;
 	char                 ** buffer;
 	char                  * name;
 	char                  * group;
 	FILE                  * logfile;
-	FILE                  * logline;
+	FILE                  * lognext_file;
+	char                  * lognext_name;
+	FILE                  * logline_file;
+	char                  * logline_name;
 	struct listroot       * list[LIST_MAX];
-	struct listroot       * history;
 	int                     rows;
 	int                     cols;
 	int                     top_row;
@@ -630,10 +692,10 @@ struct session
 	int                     socket;
 	int                     telopts;
 	int                     telopt_flag[8];
-	int                     flags;
-	char                  * host;
-	char                  * ip;
-	char                  * port;
+	long long               flags;
+	char                  * session_host;
+	char                  * session_ip;
+	char                  * session_port;
 	char                  * cmd_color;
 	unsigned char         * read_buf;
 	int                     read_len;
@@ -673,6 +735,7 @@ struct tintin_data
 	int                     input_hid;
 	int                     input_tab;
 	char                  * home;
+	char                  * lang;
 	char                  * term;
 	long long               time;
 	long long               timer[TIMER_CPU][5];
@@ -685,9 +748,10 @@ struct tintin_data
 	int                     command_ref[26];
 	int                     flags;
 	int                     quiet;
-	int                     noise_level;
 	int                     debug_level;
 	int                     input_level;
+	int                     verbose_level;
+	int                     verbatim_level;
 	char                    tintin_char;
 	char                    verbatim_char;
 	char                    repeat_char;
@@ -723,6 +787,20 @@ struct chat_data
 	long long               file_start_time;
 };
 
+struct port_data
+{
+	struct port_data      * next;
+	struct port_data      * prev;
+	char                  * name;
+	char                  * ip;
+	char                  * prefix;
+	char                  * color;
+	char                  * group;
+	int                     port;
+	int                     fd;
+	int                     flags;
+};
+
 struct link_data
 {
 	struct link_data     * next;
@@ -744,6 +822,7 @@ typedef struct session *COMMAND (struct session *ses, char *arg);
 typedef void            MAP     (struct session *ses, char *arg, char *arg1, char *arg2);
 typedef void            CURSOR  (struct session *ses, char *arg);
 typedef void            PATH    (struct session *ses, char *arg);
+typedef struct session *PORT    (struct session *ses, char *left, char *right, char *arg);
 typedef struct session *LINE    (struct session *ses, char *arg);
 typedef void            HISTORY (struct session *ses, char *arg);
 typedef void            BUFFER  (struct session *ses, char *arg);
@@ -811,6 +890,15 @@ struct list_type
 	int                     flags;
 };
 
+struct port_type
+{
+	char                  * name;
+	PORT                  * fun;
+	int                     lval;
+	int                     rval;
+	char                  * desc;
+};
+
 struct substitution_type
 {
 	char                  * name;
@@ -910,6 +998,7 @@ struct map_data
 	char                  * here_color;
 	char                  * path_color;
 	char                  * room_color;
+	char                  * back_color;
 	int                     max_grid_x;
 	int                     max_grid_y;
 	int                     undo_size;
@@ -920,7 +1009,7 @@ struct map_data
 	int                     last_room;
 	short                   display_stamp;
 	short                   nofollow;
-	char                    legend[17][100];
+	char                    legend[128][LEGEND_SIZE];
 };
 
 struct room_data
@@ -952,6 +1041,7 @@ struct exit_data
 	int                       vnum;
 	int                       dir;
 	int                       flags;
+	float                     weight;
 	char                    * name;
 	char                    * cmd;
 	char                    * data;
@@ -1002,6 +1092,8 @@ extern DO_COMMAND(do_advertise);
 extern DO_COMMAND(do_cursor);
 
 extern DO_CURSOR(cursor_backspace);
+extern DO_CURSOR(cursor_brace_open);
+extern DO_CURSOR(cursor_brace_close);
 extern DO_CURSOR(cursor_buffer_down);
 extern DO_CURSOR(cursor_buffer_end);
 extern DO_CURSOR(cursor_buffer_home);
@@ -1028,6 +1120,7 @@ extern DO_CURSOR(cursor_history_next);
 extern DO_CURSOR(cursor_history_prev);
 extern DO_CURSOR(cursor_history_search);
 extern DO_CURSOR(cursor_home);
+extern DO_CURSOR(cursor_info);
 extern DO_CURSOR(cursor_insert);
 extern DO_CURSOR(cursor_left);
 extern DO_CURSOR(cursor_left_word);
@@ -1038,7 +1131,7 @@ extern DO_CURSOR(cursor_right);
 extern DO_CURSOR(cursor_right_word);
 extern DO_CURSOR(cursor_set);
 extern DO_CURSOR(cursor_suspend);
-extern DO_CURSOR(cursor_test);
+
 extern DO_CURSOR(cursor_tab_backward);
 extern DO_CURSOR(cursor_tab_forward);
 extern DO_CURSOR(cursor_auto_tab_backward);
@@ -1074,6 +1167,7 @@ extern DO_ARRAY(array_insert);
 extern DO_ARRAY(array_delete);
 extern DO_ARRAY(array_find);
 extern DO_ARRAY(array_get);
+extern DO_ARRAY(array_simplify);
 extern DO_ARRAY(array_size);
 extern DO_ARRAY(array_set);
 extern DO_ARRAY(array_sort);
@@ -1268,12 +1362,13 @@ extern void show_vtmap(struct session *ses);
 #define __TT_MATH_H__
 
 extern DO_COMMAND(do_math);
+extern int is_math(struct session *ses, char *str);
 extern double get_number(struct session *ses, char *str);
 extern double get_double(struct session *ses, char *str);
 extern void get_number_string(struct session *ses, char *str, char *result);
 extern double mathswitch(struct session *ses, char *left, char *right);
 extern void mathexp(struct session *ses, char *str, char *result, int seed);
-extern int mathexp_tokenize(struct session *ses, char *str, int seed);
+extern int mathexp_tokenize(struct session *ses, char *str, int seed, int debug);
 extern void mathexp_level(struct session *ses, struct link_data *node);
 extern void mathexp_compute(struct session *ses, struct link_data *node);
 extern double tintoi(char *str);
@@ -1345,7 +1440,10 @@ extern DO_CONFIG(config_colorpatch);
 extern DO_CONFIG(config_mccp);
 extern DO_CONFIG(config_autotab);
 extern DO_CONFIG(config_charset);
-extern DO_CONFIG(config_256color);
+extern DO_CONFIG(config_colormode);
+extern DO_CONFIG(config_screenreader);
+extern DO_CONFIG(config_inheritance);
+extern DO_CONFIG(config_randomseed);
 
 #endif
 
@@ -1363,6 +1461,7 @@ extern void macro_update(void);
 #ifndef __STRHASH_H__
 #define __STRHASH_H__
 
+extern unsigned int generate_hash_key(char *str);
 extern char *str_hash(char *str, int lines);
 extern char *str_unhash(char *str);
 extern unsigned short str_hash_lines(char *str);
@@ -1405,7 +1504,7 @@ extern struct listnode *insert_node_list(struct listroot *root, char *ltext, cha
 extern struct listnode *update_node_list(struct listroot *root, char *ltext, char *rtext, char *prtext);
 extern struct listnode *insert_index_list(struct listroot *root, struct listnode *node, int index);
 
-extern  int show_node_with_wild(struct session *ses, char *cptr, int type);
+extern  int show_node_with_wild(struct session *ses, char *cptr, struct listroot *root);
 extern void show_node(struct listroot *root, struct listnode *node, int level);
 extern void show_nest(struct listnode *node, char *result);
 extern void show_list(struct listroot *root, int level);
@@ -1539,6 +1638,7 @@ extern DO_LINE(line_logverbatim);
 extern DO_LINE(line_quiet);
 extern DO_LINE(line_strip);
 extern DO_LINE(line_substitute);
+extern DO_LINE(line_verbatim);
 extern DO_LINE(line_verbose);
 
 #endif
@@ -1549,7 +1649,7 @@ extern DO_LINE(line_verbose);
 
 extern void logit(struct session *ses, char *txt, FILE *file, int newline);
 extern DO_COMMAND(do_log);
-extern void write_html_header(FILE *fp);
+extern void write_html_header(struct session *ses, FILE *fp);
 extern void vt102_to_html(struct session *ses, char *txt, char *out);
 #endif
 
@@ -1636,8 +1736,10 @@ extern int search_nest_index(struct listroot *root, char *variable);
 extern struct listroot *update_nest_root(struct listroot *root, char *arg);
 extern void update_nest_node(struct listroot *root, char *arg);
 extern int delete_nest_node(struct listroot *root, char *variable);
-extern int get_nest_size(struct listroot *root, char *variable, char **result);
-extern struct listnode *get_nest_node(struct listroot *root, char *variable, char **result, int def);
+extern int get_nest_size_key(struct listroot *root, char *variable, char **result);
+extern int get_nest_size_val(struct listroot *root, char *variable, char **result);
+extern struct listnode *get_nest_node_key(struct listroot *root, char *variable, char **result, int def);
+extern struct listnode *get_nest_node_val(struct listroot *root, char *variable, char **result, int def);
 extern int get_nest_index(struct listroot *root, char *variable, char **result, int def);
 extern void show_nest_node(struct listnode *node, char **result, int initialize);
 extern struct listnode *set_nest_node(struct listroot *root, char *arg1, char *format, ...);
@@ -1744,6 +1846,42 @@ extern void check_insert_path(char *command, struct session *ses);
 
 #endif
 
+#ifndef __PORT_H__
+#define __PORT_H__
+
+
+extern DO_COMMAND(do_port);
+extern DO_PORT(port_call);
+extern DO_PORT(port_color);
+extern DO_PORT(port_dnd);
+extern DO_PORT(port_group);
+extern DO_PORT(port_ignore);
+extern DO_PORT(port_initialize);
+extern DO_PORT(port_info);
+extern DO_PORT(port_message);
+extern DO_PORT(port_name);
+extern DO_PORT(port_prefix);
+extern DO_PORT(port_send);
+extern DO_PORT(port_uninitialize);
+extern DO_PORT(port_who);
+extern DO_PORT(port_zap);
+
+extern  int port_new(struct session *ses, int s);
+extern void close_port(struct session *ses, struct port_data *buddy, int unlink);
+extern void process_port_connections(struct session *ses, fd_set *read_set, fd_set *write_set, fd_set *exc_set);
+extern void port_forward_session(struct session *ses, char *linelog);
+extern void port_socket_printf(struct session *ses, struct port_data *buddy, char *format, ...);
+extern void port_printf(struct session *ses, char *format, ...);
+extern  int process_port_input(struct session *ses, struct port_data *buddy);
+extern void get_port_commands(struct session *ses, struct port_data *buddy, char *buf, int len);
+extern void port_name_change(struct session *ses, struct port_data *buddy, char *txt);
+extern void port_receive_message(struct session *ses, struct port_data *buddy, char *txt);
+
+extern void port_puts(struct session *ses, char *arg);
+extern struct port_data *find_port_buddy(struct session *ses, char *arg);
+extern struct port_data *find_port_group(struct session *ses, char *arg);
+
+#endif
 
 #ifndef __RLTAB_H__
 #define __RLTAB_H__
@@ -1816,6 +1954,7 @@ extern struct substitution_type substitution_table[];
 extern struct map_type map_table[];
 extern struct timer_type timer_table[];
 extern struct path_type path_table[];
+extern struct port_type port_table[];
 extern struct line_type line_table[];
 extern struct history_type history_table[];
 extern struct buffer_type buffer_table[];
@@ -1846,6 +1985,8 @@ extern DO_COMMAND(do_undelay);
 #ifndef __TOKENIZE_H__
 #define __TOKENIZE_H__
 
+extern void init_local(struct session *ses);
+extern struct listroot *local_list(struct session *ses);
 extern struct session *script_driver(struct session *ses, int list, char *str);
 extern char *script_writer(struct session *ses, char *str);
 
@@ -1858,6 +1999,7 @@ extern void mainloop(void);
 extern void poll_input(void);
 extern void poll_sessions(void);
 extern void poll_chat(void);
+extern void poll_port(void);
 extern void tick_update(void);
 extern void delay_update(void);
 extern void packet_update(void);
@@ -1875,6 +2017,8 @@ extern int is_color_code(char *str);
 extern int is_number(char *str);
 extern int hex_number(char *str);
 extern int oct_number(char *str);
+extern int unicode_16_bit(char *str, char *out);
+extern int unicode_21_bit(char *str, char *out);
 extern long long utime(void);
 extern char *capitalize(char *str);
 extern char *ntos(long long number);
@@ -1910,6 +2054,8 @@ extern void close_timer(int timer);
 extern DO_COMMAND(do_variable);
 extern DO_COMMAND(do_unvariable);
 
+extern DO_COMMAND(do_local);
+
 extern int delete_variable(struct session *ses, char *variable);
 extern struct listnode *search_variable(struct session *ses, char *variable);
 
