@@ -27,6 +27,7 @@
 
 #include "tintin.h"
 
+
 struct scriptdata
 {
 	long long              min;
@@ -66,9 +67,14 @@ struct scriptroot
 	struct scriptnode    * next;
 	struct scriptnode    * prev;
 	struct session       * ses;
+	struct listroot      * local;
 	int list;
 };
 
+struct scriptroot *script_stack[1001];
+
+int script_index;
+
 void debugtoken(struct session *ses, struct scriptroot *root, struct scriptnode *token)
 {
 	push_call("debugtoken(%p,%d,%p,%d)",ses,root->list,token,token->type);
@@ -84,24 +90,24 @@ void debugtoken(struct session *ses, struct scriptroot *root, struct scriptnode
 
 			case TOKEN_TYPE_ELSE:
 			case TOKEN_TYPE_END:
-				show_debug(ses, root->list, "[%02d] %*s\033[1;32m%s\033[0m", token->type, token->lvl * 4, "", token->str);
+				show_debug(ses, root->list, "[%02d] %*s\e[1;32m%s\e[0m", token->type, token->lvl * 4, "", token->str);
 				break;
 
 			case TOKEN_TYPE_DEFAULT:
-				show_debug(ses, root->list, "[%02d] %*s\033[1;32m%s\033[0m", token->type, token->lvl * 4, "", command_table[token->cmd].name);
+				show_debug(ses, root->list, "[%02d] %*s\e[1;32m%s\e[0m", token->type, token->lvl * 4, "", command_table[token->cmd].name);
 				break;
 
 			case TOKEN_TYPE_BREAK:
 			case TOKEN_TYPE_CONTINUE:
-				show_debug(ses, root->list, "[%02d] %*s\033[1;31m%s\033[0m", token->type, token->lvl * 4, "", command_table[token->cmd].name);
+				show_debug(ses, root->list, "[%02d] %*s\e[1;31m%s\e[0m", token->type, token->lvl * 4, "", command_table[token->cmd].name);
 				break;
 
 			case TOKEN_TYPE_COMMAND:
-				show_debug(ses, root->list, "[%02d] %*s\033[1;36m%s\033[0m %s\033[0m", token->type, token->lvl * 4, "", command_table[token->cmd].name, token->str);
+				show_debug(ses, root->list, "[%02d] %*s\e[1;36m%s\e[0m %s\e[0m", token->type, token->lvl * 4, "", command_table[token->cmd].name, token->str);
 				break;
 
 			case TOKEN_TYPE_RETURN:
-				show_debug(ses, root->list, "[%02d] %*s\033[1;31m%s\033[0m %s\033[0m", token->type, token->lvl * 4, "", command_table[token->cmd].name, token->str);
+				show_debug(ses, root->list, "[%02d] %*s\e[1;31m%s\e[0m %s\e[0m", token->type, token->lvl * 4, "", command_table[token->cmd].name, token->str);
 				break;
 
 			case TOKEN_TYPE_CASE:
@@ -112,11 +118,11 @@ void debugtoken(struct session *ses, struct scriptroot *root, struct scriptnode
 			case TOKEN_TYPE_PARSE:
 			case TOKEN_TYPE_SWITCH:
 			case TOKEN_TYPE_WHILE:
-				show_debug(ses, root->list, "[%02d] %*s\033[1;32m%s {\033[0m%s\033[1;32m}\033[0m", token->type, token->lvl * 4, "", command_table[token->cmd].name, token->str);
+				show_debug(ses, root->list, "[%02d] %*s\e[1;32m%s {\e[0m%s\e[1;32m}\e[0m", token->type, token->lvl * 4, "", command_table[token->cmd].name, token->str);
 				break;
 
 			case TOKEN_TYPE_REGEX:
-				show_debug(ses, root->list, "[%02d] %*s\033[1;32m%s {\033[0m%s\033[1;32m} {\033[0m%s\033[1;32m}\033[0m", token->type, token->lvl * 4, "", command_table[token->cmd].name, token->str, token->regex->str);
+				show_debug(ses, root->list, "[%02d] %*s\e[1;32m%s {\e[0m%s\e[1;32m} {\e[0m%s\e[1;32m}\e[0m", token->type, token->lvl * 4, "", command_table[token->cmd].name, token->str, token->regex->str);
 				break;
 
 			default:
@@ -126,7 +132,7 @@ void debugtoken(struct session *ses, struct scriptroot *root, struct scriptnode
 				}
 				else
 				{
-					show_debug(ses, root->list, "[%02d] %*s\033[1;33m%d {\033[0m%s\033[1;32m}\033[0m", token->type, token->lvl * 4, "", token->cmd, token->str);
+					show_debug(ses, root->list, "[%02d] %*s\e[1;33m%d {\e[0m%s\e[1;32m}\e[0m", token->type, token->lvl * 4, "", token->cmd, token->str);
 				}
 				break;
 		}
@@ -442,6 +448,32 @@ int find_command(char *command)
 	return -1;
 }
 
+void init_local(struct session *ses)
+{
+	struct scriptroot *root;
+
+	root = (struct scriptroot *) calloc(1, sizeof(struct scriptroot));
+
+	root->ses = ses;
+	root->list = LIST_VARIABLE;
+	root->local = init_list(ses, LIST_VARIABLE, LIST_SIZE);
+
+	script_stack[0] = root;
+
+	return;
+}
+
+struct listroot *local_list(struct session *ses)
+{
+	struct listroot *root;
+
+	push_call("local_list(%p)",ses);
+
+	root = script_stack[script_index]->local;
+
+	pop_call();
+	return root;
+}
 
 void tokenize_script(struct scriptroot *root, int lvl, char *str)
 {
@@ -627,6 +659,8 @@ void tokenize_script(struct scriptroot *root, int lvl, char *str)
 				}
 			}
 		}
+		str = space_out(str);
+
 		if (*str == COMMAND_SEPARATOR)
 		{
 			str++;
@@ -1058,16 +1092,21 @@ struct session *script_driver(struct session *ses, int list, char *str)
 
 	root->ses = ses;
 	root->list = list;
+	root->local = init_list(ses, LIST_VARIABLE, LIST_SIZE);
 
 	debug = HAS_BIT(ses->list[list]->flags, LIST_FLAG_DEBUG);
 
 	gtd->debug_level += debug;
 	gtd->input_level += list != LIST_COMMAND;
 
+	script_stack[++script_index] = root;
+
 	tokenize_script(root, 0, str);
 
 	ses = (struct session *) parse_script(root, 0, root->next, root->prev);
 
+	script_index--;
+
 	gtd->debug_level -= debug;
 	gtd->input_level -= list != LIST_COMMAND;
 
@@ -1076,6 +1115,7 @@ struct session *script_driver(struct session *ses, int list, char *str)
 		deltoken(root, root->prev);
 	}
 
+	free_list(root->local);
 	free(root);
 
 	pop_call();
