@@ -41,6 +41,7 @@ void mainloop(void)
 	short int pulse_poll_input      = 0 + PULSE_POLL_INPUT;
 	short int pulse_poll_sessions   = 0 + PULSE_POLL_SESSIONS;
 	short int pulse_poll_chat       = 0 + PULSE_POLL_CHAT;
+	short int pulse_poll_port       = 0 + PULSE_POLL_PORT;
 	short int pulse_update_ticks    = 0 + PULSE_UPDATE_TICKS;
 	short int pulse_update_delays   = 0 + PULSE_UPDATE_DELAYS;
 	short int pulse_update_packets  = 0 + PULSE_UPDATE_PACKETS;
@@ -80,6 +81,13 @@ void mainloop(void)
 			poll_chat();
 		}	
 
+		if (--pulse_poll_port == 0)
+		{
+			pulse_poll_port = PULSE_POLL_PORT;
+
+			poll_port();
+		}	
+
 		if (--pulse_update_ticks == 0)
 		{
 			pulse_update_ticks = PULSE_UPDATE_TICKS;
@@ -293,6 +301,51 @@ void poll_chat(void)
 	close_timer(TIMER_POLL_CHAT);
 }
 
+void poll_port(void)
+{
+	struct session *ses;
+	fd_set readfds, writefds, excfds;
+	static struct timeval to;
+	struct port_data *buddy;
+	int rv;
+
+	open_timer(TIMER_POLL_PORT);
+
+	for (ses = gts->next ; ses ; ses = gtd->update)
+	{
+		gtd->update = ses->next;
+
+		if (ses->port)
+		{
+			FD_ZERO(&readfds);
+			FD_ZERO(&writefds);
+			FD_ZERO(&excfds);
+
+			FD_SET(ses->port->fd, &readfds);
+
+			for (buddy = ses->port->next ; buddy ; buddy = buddy->next)
+			{
+				FD_SET(buddy->fd, &readfds);
+				FD_SET(buddy->fd, &writefds);
+				FD_SET(buddy->fd, &excfds);
+			}
+
+			rv = select(FD_SETSIZE, &readfds, &writefds, &excfds, &to);
+
+			if (rv <= 0)
+			{
+				if (rv == 0 || errno == EINTR)
+				{
+					return;
+				}
+				syserr("select");
+			}
+			process_port_connections(ses, &readfds, &writefds, &excfds);
+		}
+	}
+	close_timer(TIMER_POLL_PORT);
+}
+
 void tick_update(void)
 {
 	struct session *ses;
@@ -439,6 +492,7 @@ void chat_update(void)
 	close_timer(TIMER_UPDATE_CHAT);
 }
 
+
 void terminal_update(void)
 {
 	struct session *ses;
@@ -507,6 +561,10 @@ void time_update(void)
 
 	if (min[0] == old_min[0] && min[1] == old_min[1])
 	{
+		if (sec[0] == old_sec[0] && sec[1] == old_sec[1])
+		{
+			goto time_event_end;
+		}
 		goto time_event_sec;
 	}
 
@@ -549,14 +607,14 @@ void time_update(void)
 
 	strcpy(old_yrs, yrs);
 
-	check_all_events(NULL, SUB_ARG|SUB_SEC, 0, 7, "YEAR", yrs, mon, wks, day, hrs, min, sec);
-	check_all_events(NULL, SUB_ARG|SUB_SEC, 1, 7, "YEAR %s", yrs, yrs, mon, wks, day, hrs, min, sec);
+	check_all_events(NULL, SUB_ARG, 0, 7, "YEAR", yrs, mon, wks, day, hrs, min, sec);
+	check_all_events(NULL, SUB_ARG, 1, 7, "YEAR %s", yrs, yrs, mon, wks, day, hrs, min, sec);
 
 
 	time_event_mon:
 
-	check_all_events(NULL, SUB_ARG|SUB_SEC, 0, 7, "MONTH", yrs, mon, wks, day, hrs, min, sec);
-	check_all_events(NULL, SUB_ARG|SUB_SEC, 1, 7, "MONTH %s", mon, yrs, mon, wks, day, hrs, min, sec);
+	check_all_events(NULL, SUB_ARG, 0, 7, "MONTH", yrs, mon, wks, day, hrs, min, sec);
+	check_all_events(NULL, SUB_ARG, 1, 7, "MONTH %s", mon, yrs, mon, wks, day, hrs, min, sec);
 
 
 	time_event_day:
@@ -565,38 +623,42 @@ void time_update(void)
 	{
 		strcpy(old_wks, wks);
 
-		check_all_events(NULL, SUB_ARG|SUB_SEC, 0, 7, "WEEK", yrs, mon, wks, day, hrs, min, sec);
-		check_all_events(NULL, SUB_ARG|SUB_SEC, 1, 7, "WEEK %s", wks, yrs, mon, wks, day, hrs, min, sec);
+		check_all_events(NULL, SUB_ARG, 0, 7, "WEEK", yrs, mon, wks, day, hrs, min, sec);
+		check_all_events(NULL, SUB_ARG, 1, 7, "WEEK %s", wks, yrs, mon, wks, day, hrs, min, sec);
 	}
 
-	check_all_events(NULL, SUB_ARG|SUB_SEC, 2, 7, "DATE %s-%s", mon, day, yrs, mon, wks, day, hrs, min, sec);
+	check_all_events(NULL, SUB_ARG, 2, 7, "DATE %s-%s", mon, day, yrs, mon, wks, day, hrs, min, sec);
 
-	check_all_events(NULL, SUB_ARG|SUB_SEC, 0, 7, "DAY", yrs, mon, wks, day, hrs, min, sec);
-	check_all_events(NULL, SUB_ARG|SUB_SEC, 1, 7, "DAY %s", day, yrs, mon, wks, day, hrs, min, sec);
+	check_all_events(NULL, SUB_ARG, 0, 7, "DAY", yrs, mon, wks, day, hrs, min, sec);
+	check_all_events(NULL, SUB_ARG, 1, 7, "DAY %s", day, yrs, mon, wks, day, hrs, min, sec);
 
 
 	time_event_hrs:
 
-	check_all_events(NULL, SUB_ARG|SUB_SEC, 0, 7, "HOUR", yrs, mon, wks, day, hrs, min, sec);
-	check_all_events(NULL, SUB_ARG|SUB_SEC, 1, 7, "HOUR %s", hrs, yrs, mon, wks, day, hrs, min, sec);
+	check_all_events(NULL, SUB_ARG, 0, 7, "HOUR", yrs, mon, wks, day, hrs, min, sec);
+	check_all_events(NULL, SUB_ARG, 1, 7, "HOUR %s", hrs, yrs, mon, wks, day, hrs, min, sec);
 
 
 	time_event_min:
 
-	check_all_events(NULL, SUB_ARG|SUB_SEC, 4, 7, "DATE %s-%s %s:%s", mon, day, hrs, min, yrs, mon, wks, day, hrs, min, sec);
+	check_all_events(NULL, SUB_ARG, 4, 7, "DATE %s-%s %s:%s", mon, day, hrs, min, yrs, mon, wks, day, hrs, min, sec);
 
-	check_all_events(NULL, SUB_ARG|SUB_SEC, 2, 7, "TIME %s:%s", hrs, min, yrs, mon, wks, day, hrs, min, sec);
+	check_all_events(NULL, SUB_ARG, 2, 7, "TIME %s:%s", hrs, min, yrs, mon, wks, day, hrs, min, sec);
 
-	check_all_events(NULL, SUB_ARG|SUB_SEC, 0, 7, "MINUTE", yrs, mon, wks, day, hrs, min, sec);
-	check_all_events(NULL, SUB_ARG|SUB_SEC, 1, 7, "MINUTE %s", min, yrs, mon, wks, day, hrs, min, sec);
+	check_all_events(NULL, SUB_ARG, 0, 7, "MINUTE", yrs, mon, wks, day, hrs, min, sec);
+	check_all_events(NULL, SUB_ARG, 1, 7, "MINUTE %s", min, yrs, mon, wks, day, hrs, min, sec);
 
 
 	time_event_sec:
 
-	check_all_events(NULL, SUB_ARG|SUB_SEC, 3, 7, "TIME %s:%s:%s", hrs, min, sec, yrs, mon, wks, day, hrs, min, sec);
+	strcpy(old_sec, sec);
 
-	check_all_events(NULL, SUB_ARG|SUB_SEC, 0, 7, "SECOND", yrs, mon, wks, day, hrs, min, sec);
-	check_all_events(NULL, SUB_ARG|SUB_SEC, 1, 7, "SECOND %s", sec, yrs, mon, wks, day, hrs, min, sec);
+	check_all_events(NULL, SUB_ARG, 3, 7, "TIME %s:%s:%s", hrs, min, sec, yrs, mon, wks, day, hrs, min, sec);
+
+	check_all_events(NULL, SUB_ARG, 0, 7, "SECOND", yrs, mon, wks, day, hrs, min, sec);
+	check_all_events(NULL, SUB_ARG, 1, 7, "SECOND %s", sec, yrs, mon, wks, day, hrs, min, sec);
+
+	time_event_end:
 
 	close_timer(TIMER_UPDATE_TIME);
 }
