@@ -30,7 +30,7 @@
 
 DO_COMMAND(do_variable)
 {
-	char arg1[BUFFER_SIZE], arg2[BUFFER_SIZE], *str;
+	char arg1[BUFFER_SIZE], *str;
 	struct listroot *root = ses->list[LIST_VARIABLE];
 	struct listnode *node;
 
@@ -48,7 +48,7 @@ DO_COMMAND(do_variable)
 		{
 			show_node(root, node, 0);
 		}
-		else if (show_node_with_wild(ses, arg1, LIST_VARIABLE) == FALSE)
+		else if (show_node_with_wild(ses, arg1, ses->list[LIST_VARIABLE]) == FALSE)
 		{
 			show_message(ses, LIST_VARIABLE, "#VARIABLE: NO MATCH(ES) FOUND FOR {%s}.", arg1);
 		}
@@ -65,7 +65,7 @@ DO_COMMAND(do_variable)
 		{
 			arg = sub_arg_in_braces(ses, arg, str, GET_ALL, SUB_VAR|SUB_FUN);
 
-			if (*arg2)
+			if (*str)
 			{
 				add_nest_node(root, arg1, "%s", str);
 			}
@@ -80,24 +80,83 @@ DO_COMMAND(do_variable)
 	return ses;
 }
 
-DO_COMMAND(do_unvariable)
+DO_COMMAND(do_local)
 {
-	char arg1[BUFFER_SIZE], arg2[BUFFER_SIZE];
+	char arg1[BUFFER_SIZE], *str;
+	struct listroot *root;
+	struct listnode *node;
 
-	sub_arg_in_braces(ses, arg, arg1, GET_ONE, SUB_VAR|SUB_FUN);
-	sub_arg_in_braces(ses, arg, arg2, GET_ALL, SUB_VAR|SUB_FUN);
+	root = local_list(ses);
 
-	if (delete_nest_node(ses->list[LIST_VARIABLE], arg2))
+	arg = sub_arg_in_braces(ses, arg, arg1, GET_NST, SUB_VAR|SUB_FUN);
+
+	if (*arg1 == 0)
 	{
-		show_message(ses, LIST_VARIABLE, "#OK. {%s} IS NO LONGER A VARIABLE.", arg2);
+		show_list(root, 0);
+	}
+	else if (*arg1 && *arg == 0)
+	{
+		node = search_nest_node(root, arg1);
+
+		if (node)
+		{
+			show_node(root, node, 0);
+		}
+		else if (show_node_with_wild(ses, arg1, root) == FALSE)
+		{
+			show_message(ses, LIST_VARIABLE, "#LOCAL: NO MATCH(ES) FOUND FOR {%s}.", arg1);
+		}
 	}
 	else
 	{
-		delete_node_with_wild(ses, LIST_VARIABLE, arg1);
+		str = str_alloc(UMAX(strlen(arg), BUFFER_SIZE));
+
+		arg = sub_arg_in_braces(ses, arg, str, GET_ALL, SUB_VAR|SUB_FUN);
+
+		node = set_nest_node(root, arg1, "%s", str);
+
+		while (*arg)
+		{
+			arg = sub_arg_in_braces(ses, arg, str, GET_ALL, SUB_VAR|SUB_FUN);
+
+			if (*str)
+			{
+				add_nest_node(root, arg1, "%s", str);
+			}
+		}
+
+		show_nest_node(node, &str, 1);
+
+		show_message(ses, LIST_VARIABLE, "#OK. LOCAL VARIABLE {%s} HAS BEEN SET TO {%s}.", arg1, str);
+
+		str_free(str);
 	}
 	return ses;
 }
 
+DO_COMMAND(do_unvariable)
+{
+	char arg1[BUFFER_SIZE];
+
+	arg = sub_arg_in_braces(ses, arg, arg1, GET_ALL, SUB_VAR|SUB_FUN);
+
+	do
+	{
+		if (delete_nest_node(ses->list[LIST_VARIABLE], arg1))
+		{
+			show_message(ses, LIST_VARIABLE, "#OK. {%s} IS NO LONGER A VARIABLE.", arg1);
+		}
+		else
+		{
+			delete_node_with_wild(ses, LIST_VARIABLE, arg1);
+		}
+		arg = sub_arg_in_braces(ses, arg, arg1, GET_ALL, SUB_VAR|SUB_FUN);
+	}
+	while (*arg1);
+
+	return ses;
+}
+
 
 DO_COMMAND(do_replace)
 {
@@ -105,21 +164,32 @@ DO_COMMAND(do_replace)
 	struct listroot *root;
 	struct listnode *node;
 
-	root = ses->list[LIST_VARIABLE];
-
-	arg = get_arg_in_braces(ses, arg, arg1, GET_NST);
+	arg = sub_arg_in_braces(ses, arg, arg1, GET_NST, SUB_VAR|SUB_FUN);
 	arg = sub_arg_in_braces(ses, arg, arg2, GET_ONE, SUB_VAR|SUB_FUN);
 	arg = sub_arg_in_braces(ses, arg, arg3, GET_ALL, SUB_VAR|SUB_FUN);
 
 	if (*arg1 == 0 || *arg2 == 0)
 	{
 		show_error(ses, LIST_VARIABLE, "#SYNTAX: #REPLACE {VARIABLE} {OLD TEXT} {NEW TEXT}");
+
+		return ses;
 	}
-	else if ((node = search_nest_node(root, arg1)) == NULL)
+
+	root = local_list(ses);
+
+	if ((node = search_nest_node(root, arg1)) == NULL)
 	{
-		show_error(ses, LIST_VARIABLE, "#REPLACE: VARIABLE {%s} NOT FOUND.", arg1);
+		root = ses->list[LIST_VARIABLE];
+
+		if ((node = search_nest_node(root, arg1)) == NULL)
+		{
+			show_error(ses, LIST_VARIABLE, "#REPLACE: VARIABLE {%s} NOT FOUND.", arg1);
+
+			return ses;
+		}
 	}
-	else if (tintin_regexp(ses, NULL, node->right, arg2, 0, SUB_CMD) == FALSE)
+
+	if (tintin_regexp(ses, NULL, node->right, arg2, 0, SUB_CMD) == FALSE)
 	{
 		show_message(ses, LIST_VARIABLE, "#REPLACE: {%s} NOT FOUND IN {%s}.", arg2, node->right);
 	}
@@ -154,7 +224,7 @@ DO_COMMAND(do_replace)
 
 		strcat(buf, pti);
 
-		set_nest_node(ses->list[LIST_VARIABLE], arg1, "%s", buf);
+		set_nest_node(root, arg1, "%s", buf);
 	}
 	return ses;
 }
@@ -316,17 +386,35 @@ void stripspaces(char *str)
 			break;
 		}
 	}
-	strcpy(str, &str[cnt]);
+	memcpy(str, &str[cnt], strlen(&str[cnt]) + 1);
+//	strcpy(str, &str[cnt]);
 }
 
 void wrapstring(struct session *ses, char *str)
 {
-	char *pti, *lis, *sos, *soc, buf[BUFFER_SIZE], color[BUFFER_SIZE], tmp[BUFFER_SIZE];
-	int col = 1, cnt = 1;
+	char *pti, *lis, *sos, *soc, buf[BUFFER_SIZE], color[BUFFER_SIZE], tmp[BUFFER_SIZE], left[BUFFER_SIZE], right[BUFFER_SIZE], *arg;
+	int col = 1, cnt = 1, len;
 
 	push_call("wrapstring(%p,%p)",ses,str);
+	
+	arg = get_arg_in_braces(ses, str, left, TRUE);
 
-	pti = lis = sos = soc = str;
+	if (*arg == COMMAND_SEPARATOR)
+	{
+		arg++;
+	}
+	arg = get_arg_in_braces(ses, arg, right, TRUE);
+
+	if (*right)
+	{
+		len = get_number(ses, right);
+	}
+	else
+	{
+		len = ses->cols;
+	}
+
+	pti = lis = sos = soc = left;
 
 	buf[0] = color[0] = 0;
 
@@ -344,11 +432,11 @@ void wrapstring(struct session *ses, char *str)
 			lis = pti;
 		}
 
-		if (col > ses->cols || *pti == '\n')
+		if (col > len || *pti == '\n')
 		{
 			col = 1;
 
-			if (pti - lis > 15)
+			if (pti - lis >= len || pti - lis > 15)
 			{
 				sprintf(tmp, "%.*s", (int) (sos - soc), soc);
 
@@ -375,7 +463,24 @@ void wrapstring(struct session *ses, char *str)
 		}
 		else
 		{
-			pti++;
+			if (HAS_BIT(ses->flags, SES_FLAG_BIG5) && *pti & 128 && pti[1] != 0)
+			{
+				pti++;
+				pti++;
+			}
+			else if (HAS_BIT(ses->flags, SES_FLAG_UTF8) && (*pti & 192) == 192)
+			{
+				pti++;
+
+				while ((*pti & 192) == 128)
+				{
+					pti++;
+				}
+			}
+			else
+			{
+				pti++;
+			}
 			col++;
 		}
 	}
@@ -434,6 +539,20 @@ int string_str_raw_len(struct session *ses, char *str, int start, int end)
 			break;
 		}
 
+		if (str[raw_cnt] == '\\')
+		{
+			ret_cnt += (str_cnt >= start) ? 1 : 0;
+			raw_cnt++;
+			
+			if (str[raw_cnt] == '\\')
+			{
+				ret_cnt += (str_cnt >= start) ? 1 : 0;
+				raw_cnt++;
+				str_cnt++;
+			}
+			continue;
+		}
+
 		if (HAS_BIT(ses->flags, SES_FLAG_UTF8) && (str[raw_cnt] & 192) == 192)
 		{
 			ret_cnt += (str_cnt >= start) ? 1 : 0;
@@ -455,6 +574,8 @@ int string_str_raw_len(struct session *ses, char *str, int start, int end)
 	return ret_cnt;
 }
 
+// Returns the stripped length for the range
+
 int string_raw_str_len(struct session *ses, char *str, int start, int end)
 {
 	int raw_cnt, ret_cnt, tot_len;
@@ -484,6 +605,18 @@ int string_raw_str_len(struct session *ses, char *str, int start, int end)
 			continue;
 		}
 
+		if (str[raw_cnt] == '\\')
+		{
+			raw_cnt++;
+
+			if (str[raw_cnt] == '\\')
+			{
+				raw_cnt++;
+				ret_cnt++;
+			}
+			continue;
+		}
+
 		if (HAS_BIT(gtd->ses->flags, SES_FLAG_UTF8) && (str[raw_cnt] & 192) == 192)
 		{
 			raw_cnt++;
@@ -531,14 +664,15 @@ void timestring(struct session *ses, char *str)
 	strftime(str, BUFFER_SIZE, left, &timeval_tm);
 }
 
+
 void format_string(struct session *ses, char *format, char *arg, char *out)
 {
-	char temp[BUFFER_SIZE], newformat[BUFFER_SIZE], arglist[20][BUFFER_SIZE], *ptf, *ptt, *pts, *ptn;
+	char temp[BUFFER_SIZE], newformat[BUFFER_SIZE], arglist[30][20000], *ptf, *ptt, *pts, *ptn;
 	struct tm timeval_tm;
 	time_t    timeval_t;
 	int i;
 
-	for (i = 0 ; i < 20 ; i++)
+	for (i = 0 ; i < 30 ; i++)
 	{
 		arg = sub_arg_in_braces(ses, arg, arglist[i], GET_ONE, SUB_VAR|SUB_FUN);
 	}
@@ -550,7 +684,7 @@ void format_string(struct session *ses, char *format, char *arg, char *out)
 
 	while (*ptf)
 	{
-		if (i == 20)
+		if (i == 30)
 		{
 			break;
 		}
@@ -734,6 +868,10 @@ void format_string(struct session *ses, char *format, char *arg, char *out)
 						thousandgroupingstring(ses, arglist[i]);
 						break;
 
+					case 'H':
+						sprintf(arglist[i], "%u", generate_hash_key(arglist[i]));
+						break;
+
 					case 'L':
 						sprintf(arglist[i], "%d", stringlength(ses, arglist[i]));
 						break;
@@ -748,6 +886,10 @@ void format_string(struct session *ses, char *format, char *arg, char *out)
 						sprintf(arglist[i], "%d", ses->rows);
 						break;
 
+					case 'S':
+						sprintf(arglist[i], "%s", ses->name);
+						break;
+
 					case 'T':
 						sprintf(arglist[i], "%d", (int) time(NULL));
 						break;
@@ -778,7 +920,7 @@ void format_string(struct session *ses, char *format, char *arg, char *out)
 	}
 	*ptn = 0;
 
-	sprintf(out, newformat, arglist[0], arglist[1], arglist[2], arglist[3], arglist[4], arglist[5], arglist[6], arglist[7], arglist[8], arglist[9], arglist[10], arglist[11], arglist[12], arglist[13], arglist[14], arglist[15], arglist[16], arglist[17], arglist[18], arglist[19]);
+	sprintf(out, newformat, arglist[0], arglist[1], arglist[2], arglist[3], arglist[4], arglist[5], arglist[6], arglist[7], arglist[8], arglist[9], arglist[10], arglist[11], arglist[12], arglist[13], arglist[14], arglist[15], arglist[16], arglist[17], arglist[18], arglist[19], arglist[20], arglist[21], arglist[22], arglist[23], arglist[24], arglist[25], arglist[26], arglist[27], arglist[28], arglist[29]);
 
 	return;
 }
